Package: BiplotML
Title: Logistic Biplot Estimation Using Machine Learning Algorithms
Version: 1.1.1
Date: 2026-04-30
Authors@R: 
    person(given = "Jose Giovany",
           family = "Babativa-Marquez",
           role = c("cre", "aut"),
           email = "jgbabativam@unal.edu.co",
           comment = c(ORCID = "0000-0002-4989-7459"))
Description: Implements methods for fitting logistic biplot models to
    multivariate binary data. The logistic biplot represents individuals
    as points and binary variables as directed vectors in a low-dimensional
    subspace; the orthogonal projection of each individual onto a variable
    vector approximates the expected probability that the corresponding
    characteristic is present. Available fitting methods include conjugate
    gradient algorithms, a coordinate descent Majorization-Minimization
    (MM) algorithm, and a block coordinate descent algorithm based on data
    projection that supports matrices with missing values and allows new
    individuals to be projected as supplementary rows without refitting
    the model. A cross-validation procedure is provided to select the
    number of latent dimensions k.
    References: Babativa-Marquez and Vicente-Villardon (2021)
    <doi:10.3390/math9162015>;
    Vicente-Villardon and Galindo (2006, ISBN:9780470973196).
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Imports: optimx, RSpectra
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, dplyr (>= 1.0.0),
        tidyr (>= 1.1.0), ggplot2 (>= 3.3.2), ggrepel, pracma, mvtnorm
URL: https://github.com/jgbabativam/BiplotML
BugReports: https://github.com/jgbabativam/BiplotML/issues
NeedsCompilation: no
Packaged: 2026-05-03 14:09:14 UTC; giova
Author: Jose Giovany Babativa-Marquez [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-4989-7459>)
Maintainer: Jose Giovany Babativa-Marquez <jgbabativam@unal.edu.co>
Repository: CRAN
Date/Publication: 2026-05-08 15:42:17 UTC
Built: R 4.5.2; ; 2026-05-08 20:17:50 UTC; unix

Package: BiplotML
Title: Logistic Biplot Estimation Using Machine Learning Algorithms
Version: 1.1.1
Date: 2026-04-30
Authors@R: 
    person(given = "Jose Giovany",
           family = "Babativa-Marquez",
           role = c("cre", "aut"),
           email = "jgbabativam@unal.edu.co",
           comment = c(ORCID = "0000-0002-4989-7459"))
Description: Implements methods for fitting logistic biplot models to
    multivariate binary data. The logistic biplot represents individuals
    as points and binary variables as directed vectors in a low-dimensional
    subspace; the orthogonal projection of each individual onto a variable
    vector approximates the expected probability that the corresponding
    characteristic is present. Available fitting methods include conjugate
    gradient algorithms, a coordinate descent Majorization-Minimization
    (MM) algorithm, and a block coordinate descent algorithm based on data
    projection that supports matrices with missing values and allows new
    individuals to be projected as supplementary rows without refitting
    the model. A cross-validation procedure is provided to select the
    number of latent dimensions k.
    References: Babativa-Marquez and Vicente-Villardon (2021)
    <doi:10.3390/math9162015>;
    Vicente-Villardon and Galindo (2006, ISBN:9780470973196).
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Imports: optimx, RSpectra
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, dplyr (>= 1.0.0),
        tidyr (>= 1.1.0), ggplot2 (>= 3.3.2), ggrepel, pracma, mvtnorm
URL: https://github.com/jgbabativam/BiplotML
BugReports: https://github.com/jgbabativam/BiplotML/issues
NeedsCompilation: no
Packaged: 2026-05-03 14:09:14 UTC; giova
Author: Jose Giovany Babativa-Marquez [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-4989-7459>)
Maintainer: Jose Giovany Babativa-Marquez <jgbabativam@unal.edu.co>
Repository: CRAN
Date/Publication: 2026-05-08 15:42:17 UTC
Built: R 4.4.3; ; 2026-05-08 18:16:24 UTC; unix

# Generated by roxygen2: do not edit by hand

export(LogBip)
export(cv_LogBip)
export(fitted_LB)
export(gradientDesc)
export(performanceBLB)
export(plotBLB)
export(pred_LB)
export(proj_LogBip)
export(sdv_MM)
export(simBin)
importFrom(optimx,optimr)
importFrom(optimx,optimx)
importFrom(stats,IQR)
importFrom(stats,plogis)
importFrom(stats,qlogis)
importFrom(stats,rbinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)

#' @export
#' @title Fit a Binary Logistic Biplot with Missing Data via Block Coordinate Descent
#' @description
#' Estimates the intercept vector \eqn{\mu}, the row-marker matrix \strong{A},
#' and the column-marker matrix \strong{B} using a data-projection model with a
#' block coordinate descent algorithm. Missing values in the binary matrix are
#' imputed iteratively during model fitting. This function also allows new
#' individuals to be projected as supplementary rows without refitting the model,
#' since the row markers are derived directly from the estimated column markers.
#' This is the low-level function called by \code{\link{LogBip}} when
#' \code{method = "PDLB"}.
#'
#' @author Giovany Babativa <jgbabativam@@unal.edu.co>
#'
#' @param x A binary matrix, possibly containing \code{NA} values.
#' @param k Number of dimensions. Default is \code{k = 5}.
#' @param max_iters Maximum number of iterations. Default is \code{1000}.
#' @param random_start Logical; if \code{TRUE}, parameters are initialised
#'   randomly. Default is \code{FALSE} (SVD initialisation).
#' @param epsilon Convergence tolerance for the relative decrease in the loss
#'   function. Default is \code{1e-5}.
#'
#' @return A named list with components:
#'   \describe{
#'     \item{\code{mu}}{Estimated intercept vector of length \eqn{p}.}
#'     \item{\code{A}}{Estimated row-marker matrix (\eqn{n \times k}).}
#'     \item{\code{B}}{Estimated column-marker matrix (\eqn{p \times k}).}
#'     \item{\code{x_est}}{Imputed binary matrix (missing entries replaced by
#'       fitted values).}
#'     \item{\code{iter}}{Number of iterations performed.}
#'     \item{\code{loss_funct}}{Vector of normalised loss-function values at each
#'       iteration.}
#'   }
#'
#' @references
#' Babativa-Marquez, J. G., & Vicente-Villardon, J. L. (2026). Logistic biplot
#' with missing data. \emph{In process}.
#'
#' Babativa-Marquez, J. G., & Vicente-Villardon, J. L. (2021). Logistic biplot
#' by conjugate gradient algorithms and iterated SVD. \emph{Mathematics},
#' \emph{9}(16), 2015. \doi{10.3390/math9162015}
#'
#' Vicente-Villardon, J. L., & Galindo, M. P. (2006). Logistic biplots. In
#' M. Greenacre & J. Blasius (Eds.), \emph{Multiple Correspondence Analysis and
#' Related Methods} (pp. 503--521). Chapman & Hall.
#'
#' @seealso \code{\link{LogBip}}, \code{\link{cv_LogBip}}
#'
#' @examples
#' \donttest{
#' data("Methylation")
#' set.seed(12345)
#' n <- nrow(Methylation); p <- ncol(Methylation)
#' miss <- matrix(rbinom(n * p, 1, 0.2), n, p)
#' miss <- ifelse(miss == 1, NA, miss)
#' x_miss <- Methylation + miss
#' out <- proj_LogBip(x = x_miss, k = 2, max_iters = 1000)
#' }

proj_LogBip <- function(x, k = 5, max_iters = 1000,
                        random_start = FALSE, epsilon = 1e-05) {

  x <- as.matrix(x)

  verify <- apply(x, 2, sd, na.rm = TRUE)
  if (any(verify == 0)) {
    stop("Some variables have zero variance; the procedure cannot be applied.")
  }

  n <- nrow(x); p <- ncol(x)

  partial_decomp <- FALSE
  if (n > 100 | p > 20) {
    partial_decomp <- TRUE
    if (!requireNamespace("RSpectra", quietly = TRUE)) {
      message("Package 'RSpectra' is recommended to speed up the algorithm.")
      partial_decomp <- FALSE
    }
  }

  if (k > p) {
    warning("k must be less than or equal to p. Setting k = p.")
    k <- p
  }

  W  <- ifelse(!is.na(x), 1, 0)
  xi <- ifelse(is.na(x), 1, x)
  x0 <- W * xi + 0.5 * (1 - W)

  if (random_start) {
    b0 <- rep(0, p)
    V  <- matrix(rnorm(p * k), p, k)
  } else {
    b0 <- colMeans(x0, na.rm = TRUE)
    if (partial_decomp) {
      udv <- RSpectra::svds(scale(x0, center = b0, scale = FALSE), k = k)
    } else {
      udv <- svd(scale(x0, center = b0, scale = FALSE))
    }
    V <- matrix(udv$v[, 1:k], p, k)
  }

  A     <- (x0 - rep(1, n) %*% t(b0)) %*% V
  B     <- V
  theta <- rep(1, n) %*% t(b0) + A %*% t(B)
  P     <- plogis(theta)

  thresh <- thresholds(x0, P, ncuts = 100)$thres

  err    <- numeric(max_iters + 1)
  xl     <- x0
  err[1] <- log_like(x = xl, w = W, theta = theta) / (n * p)

  l <- 0
  for (l in seq_len(max_iters)) {
    Zl  <- theta + 4 * (xl - P)
    Ml  <- W * Zl + (1 - W) * theta
    Xlc <- xl - rep(1, n) %*% t(b0)
    Mlc <- Ml - rep(1, n) %*% t(b0)
    Yl  <- -(t(Xlc) %*% Xlc - t(Xlc) %*% Mlc - t(Mlc) %*% Xlc)

    sdv <- RSpectra::eigs_sym(Yl, k, which = "LM")
    if (any(sdv$values[1:k] < 0)) {
      sdv <- eigen(Yl, symmetric = TRUE)
    }

    V  <- matrix(sdv$vectors[, 1:k], p, k)
    b0 <- as.vector(1 / n * t(Ml - xl %*% tcrossprod(V)) %*% rep(1, n))

    theta <- rep(1, n) %*% t(b0) +
      (xl - rep(1, n) %*% t(b0)) %*% tcrossprod(V)
    P <- plogis(theta)

    alphas <- rep(1, n) %*% t(as.matrix(thresh$threshold))
    xhat   <- ifelse(P < alphas, 0, 1)
    xl     <- W * xi + (1 - W) * xhat

    err[l + 1] <- log_like(x = xl, w = W, theta = theta) /
      (nrow(x) * ncol(x))

    if (l == max_iters) {
      warning("The algorithm reached ", max_iters,
              " iterations without converging.")
    }

    if (l > 2) {
      if ((err[l] - err[l + 1]) / err[l] < epsilon) break
    }
  }

  A <- (xl - rep(1, n) %*% t(b0)) %*% V
  B <- V

  out <- list(mu = b0, A = A, B = B, x_est = xl,
              iter = l, loss_funct = err[seq_len(l)])
  return(out)
}

# Package-level declarations
# This file is loaded last (alphabetically) and contains
# internal declarations that must not be processed as
# documentation blocks by roxygen2.

# Suppress R CMD check NOTEs for ggplot2 tidy-evaluation column
# names used inside aes() in plotBLB().
utils::globalVariables(c(
  ".data", "label",
  "x.50",  "y.50",
  "x.end", "y.end",
  "ind",   "resample",
  "Dim1",  "Dim2",
  "Dimb1", "Dimb2",
  "b0",    "b1",    "b2",
  "bb0",   "bb1",   "bb2",
  "Dim1c", "Dim2c"
))

#' @export
#' @title Fit a Binary Logistic Biplot via Coordinate Descent MM Algorithm
#' @description
#' Estimates the intercept vector \eqn{\mu}, the row-marker matrix \strong{A},
#' and the column-marker matrix \strong{B} using an iterative coordinate descent
#' Majorization-Minimization (MM) algorithm. This is the low-level function
#' called by \code{\link{LogBip}} when \code{method = "MM"}.
#'
#' @author Giovany Babativa <jgbabativam@@unal.edu.co>
#'
#' @param x A binary matrix with no missing values.
#' @param k Number of dimensions. Default is \code{k = 5}.
#' @param iterations Maximum number of iterations. Default is \code{1000}.
#' @param truncated Logical; if \code{TRUE} (default for large matrices), the
#'   truncated SVD from \pkg{RSpectra} is used to speed up computation.
#' @param random Logical; if \code{TRUE}, parameters are initialised randomly.
#'   Default is \code{FALSE} (SVD initialisation).
#' @param epsilon Convergence tolerance. The algorithm stops when the relative
#'   decrease in the loss function is below this value. Default is \code{1e-4}.
#'
#' @return A named list with components:
#'   \describe{
#'     \item{\code{mu}}{Estimated intercept vector of length \eqn{p}.}
#'     \item{\code{A}}{Estimated row-marker matrix (\eqn{n \times k}).}
#'     \item{\code{B}}{Estimated column-marker matrix (\eqn{p \times k}).}
#'     \item{\code{iterations}}{Number of iterations performed.}
#'     \item{\code{loss_func}}{Vector of normalised loss-function values at each
#'       iteration.}
#'   }
#'
#' @references
#' Babativa-Marquez, J. G., & Vicente-Villardon, J. L. (2021). Logistic biplot
#' by conjugate gradient algorithms and iterated SVD. \emph{Mathematics},
#' \emph{9}(16), 2015. \doi{10.3390/math9162015}
#'
#' Vicente-Villardon, J. L., & Galindo, M. P. (2006). Logistic biplots. In
#' M. Greenacre & J. Blasius (Eds.), \emph{Multiple Correspondence Analysis and
#' Related Methods} (pp. 503--521). Chapman & Hall.
#'
#' @seealso \code{\link{LogBip}}, \code{\link{cv_LogBip}}
#'
#' @examples
#' \donttest{
#' data("Methylation")
#' out <- sdv_MM(x = Methylation)
#' }
sdv_MM <- function(x, k = 5, iterations = 1000, truncated = TRUE,
                   random = FALSE, epsilon = 1e-4) {

  n <- nrow(x); p <- ncol(x)

  if (k > p) {
    message("k must be less than or equal to the number of columns. Setting k = p.")
    k <- p
  }

  if (!random) {
    mu <- colMeans(8 * x)
    if (truncated) {
      vp <- RSpectra::svds(scale(8 * x, center = TRUE, scale = FALSE),
                           k = k, nu = k, nv = k)
    } else {
      vp <- svd(scale(8 * x, center = TRUE, scale = FALSE))
    }
    if (k == 1) {
      A <- matrix(vp$u[, 1:k], n, k) * vp$d[1]
      B <- matrix(vp$v[, 1:k], p, k)
    } else {
      A <- vp$u[, 1:k] %*% diag(vp$d[1:k])
      B <- vp$v[, 1:k]
    }
  } else {
    mu <- runif(p)
    A  <- matrix(runif(n * k), n, k)
    B  <- matrix(runif(p * k), p, k)
  }

  theta <- rep(1, n) %*% t(mu) + (A %*% t(B))
  loss_func <- numeric(iterations + 1)
  pi_hat <- plogis(theta)
  loss_func[1] <- -sum(x * log(pi_hat) + (1 - x) * log(1 - pi_hat),
                       na.rm = TRUE) / (n * p)

  j <- 0
  for (i in seq_len(iterations)) {
    old_mu <- mu; old_A <- A; old_B <- B

    Z  <- theta + 4 * (x - pi_hat)
    mu <- colMeans(Z)

    if (truncated) {
      vp <- RSpectra::svds(scale(Z, center = TRUE, scale = FALSE),
                           k = k, nu = k, nv = k)
    } else {
      vp <- svd(scale(Z, center = TRUE, scale = FALSE))
    }

    if (k == 1) {
      A <- matrix(vp$u[, 1:k], n, k) * vp$d[1]
      B <- matrix(vp$v[, 1:k], p, k)
    } else {
      A <- vp$u[, 1:k] %*% diag(vp$d[1:k])
      B <- vp$v[, 1:k]
    }

    theta   <- rep(1, n) %*% t(mu) + (A %*% t(B))
    pi_hat  <- plogis(theta)
    loss_func[i + 1] <- -sum(x * log(pi_hat) + (1 - x) * log(1 - pi_hat),
                             na.rm = TRUE) / (n * p)

    if (loss_func[i] < loss_func[i + 1]) {
      mu <- old_mu; A <- old_A; B <- old_B
      if (truncated) {
        warning("The loss function increased at iteration ", i,
                ". Consider rerunning with truncated = FALSE.")
      }
    }

    if (i > 10) {
      if ((loss_func[i] - loss_func[i + 1]) / loss_func[i] < epsilon)
        break
    }

    if (i == iterations) {
      warning("The algorithm reached the maximum of ", iterations,
              " iterations without converging.")
    }

    j <- i
  }

  out <- list(mu = mu, A = A, B = B, iterations = j,
              loss_func = loss_func[seq_len(j)])
  return(out)
}

#' @export
#' @title Predict Binary Responses from a Logistic Biplot
#' @description
#' Predicts the binary response matrix from a fitted logistic biplot and
#' computes the optimal classification threshold for each variable by minimising
#' the Balanced Error Rate (BER).
#'
#' @details
#' The optimal threshold for variable \eqn{j} is the value \eqn{\alpha_j \in [0,1]}
#' that minimises the Balanced Error Rate:
#' \deqn{BER_j = 1 - \frac{1}{2}
#'   \left(\frac{TP_j}{TP_j + FN_j} + \frac{TN_j}{TN_j + FP_j}\right),}
#' where \eqn{TP}, \eqn{TN}, \eqn{FP}, and \eqn{FN} denote true positives,
#' true negatives, false positives, and false negatives, respectively.
#'
#' @author Giovany Babativa <jgbabativam@@unal.edu.co>
#'
#' @param object An object of class \code{BiplotML}, as returned by
#'   \code{\link{LogBip}}.
#' @param x The original binary matrix used to fit the model.
#' @param ncuts Number of equally spaced threshold candidates in \eqn{[0, 1]}.
#'   Default is \code{100}.
#'
#' @return A named list of class \code{BiplotML} with components:
#'   \describe{
#'     \item{\code{thresholds}}{Data frame with the optimal threshold and minimum
#'       BER for each variable.}
#'     \item{\code{predictX}}{Predicted binary matrix.}
#'     \item{\code{fitted}}{Confusion matrix (sensitivity, specificity, global
#'       accuracy) for each variable.}
#'     \item{\code{BER}}{Overall Balanced Error Rate (in percent).}
#'   }
#'
#' @examples
#' \donttest{
#' data("Methylation")
#' LB  <- LogBip(Methylation, plot = FALSE)
#' out <- pred_LB(LB, Methylation)
#' }

pred_LB <- function(object, x, ncuts = 100) {

  for (pkg in c("tidyr", "dplyr")) {
    if (!requireNamespace(pkg, quietly = TRUE)) {
      stop("Package '", pkg, "' is required for this function. Please install it.",
           call. = FALSE)
    }
  }

  P <- fitted_LB(object, type = "response")

  if (is.null(colnames(x))) colnames(x) <- paste0("V", seq_len(ncol(x)))

  N1 <- ceiling(sum(x, na.rm = TRUE))
  N0 <- length(as.matrix(x)) - N1

  TE <- lapply(seq(0, 1, length.out = ncuts), function(z) {
    Pr <- ifelse(P >= z, 1, 0)
    c1 <- 1 - apply((Pr == 1) & (x == 1), 2, sum, na.rm = TRUE) /
              apply(x == 1, 2, sum)
    c2 <- 1 - apply((Pr == 0) & (x == 0), 2, sum, na.rm = TRUE) /
              apply(x == 0, 2, sum)
    list(100 / 2 * (c1 + c2))
  })

  TEp <- data.frame(dplyr::bind_rows(TE),
                    threshold = seq(0, 1, length.out = ncuts))

  thresholds <- TEp |>
    tidyr::pivot_longer(-threshold, names_to = "variable",
                        values_to = "BACC") |>
    dplyr::group_by(variable) |>
    dplyr::mutate(merror = min(BACC)) |>
    dplyr::filter(BACC == merror) |>
    dplyr::mutate(row = dplyr::row_number()) |>
    dplyr::filter(row == 1) |>
    dplyr::ungroup() |>
    dplyr::select(variable, threshold, BACC)

  thresholds <- thresholds[match(colnames(x), thresholds$variable), ]

  Pr <- matrix(NA, nrow(P), ncol(P))
  for (p in seq_len(ncol(P))) {
    Pr[, p] <- ifelse(P[, p] >= thresholds$threshold[p], 1, 0)
  }

  c1   <- 1 - sum(apply((Pr == 1) & (x == 1), 2, sum, na.rm = TRUE)) / N1
  c2   <- 1 - sum(apply((Pr == 0) & (x == 0), 2, sum, na.rm = TRUE)) / N0
  BACC <- round(100 / 2 * (c1 + c2), 2)

  PCC      <- ifelse((x == 1 & Pr == 1) | (x == 0 & Pr == 0), 1, 0)
  PCC[is.na(PCC)] <- 0
  ones     <- apply(x, 2, sum, na.rm = TRUE)
  zeros    <- length(as.matrix(x)) - ones

  confusion <- data.frame(
    Sensitivity = round(100 * apply((Pr == 1) & (x == 1), 2, sum,
                                    na.rm = TRUE) / ones,  1),
    Specificity = round(100 * apply((Pr == 0) & (x == 0), 2, sum,
                                    na.rm = TRUE) / zeros, 1),
    Global      = round(100 * colSums(PCC) / nrow(PCC), 1)
  )

  out <- list(thresholds = thresholds, predictX = Pr,
              fitted = confusion, BER = BACC)
  class(out) <- c("BiplotML", "list")
  return(out)
}

#' @importFrom optimx optimx
#' @export
#' @title Compare Optimization Algorithms for Binary Logistic Biplot Estimation
#' @description
#' Fits the binary logistic biplot model using multiple optimization algorithms
#' and returns a summary of their computation time, convergence status, and
#' number of function evaluations, facilitating algorithm selection.
#'
#' @details
#' The following algorithm groups are available via the \code{method} argument:
#' \itemize{
#'   \item \code{1} --- Derivative-free methods: Nelder-Mead, UOBYQA, NEWUOA.
#'   \item \code{2} --- Gradient methods (default): CG, Rcgmin.
#'   \item \code{3} --- Quasi-Newton methods: BFGS, L-BFGS-B, nlm, nlminb.
#'   \item \code{4} --- All of the above.
#' }
#'
#' @author Giovany Babativa <jgbabativam@@unal.edu.co>
#'
#' @param xi A binary matrix.
#' @param k Number of dimensions. Default is \code{k = 2}.
#' @param L Ridge penalization parameter. Default is \code{L = 0}.
#' @param method Algorithm group to compare: \code{1} (derivative-free),
#'   \code{2} (gradient, default), \code{3} (quasi-Newton), or \code{4} (all).
#' @param maxit Maximum number of iterations per algorithm.
#'
#' @return A data frame with one row per algorithm and columns:
#'   \describe{
#'     \item{\code{method}}{Algorithm name.}
#'     \item{\code{evaluat}}{Final value of the objective function.}
#'     \item{\code{convergence}}{Convergence status.}
#'     \item{\code{fevals}}{Number of function evaluations.}
#'     \item{\code{time}}{Elapsed computation time.}
#'   }
#'
#' @references
#' Nash, J. C. (2011). Unifying optimization algorithms to aid software system
#' users: optimx for R. \emph{Journal of Statistical Software}, \emph{43}(9),
#' 1--14.
#'
#' Nash, J. C. (2014). On best practice optimization methods in R.
#' \emph{Journal of Statistical Software}, \emph{60}(2), 1--14.
#'
#' Vicente-Villardon, J. L., & Galindo, M. P. (2006). Logistic biplots. In
#' M. Greenacre & J. Blasius (Eds.), \emph{Multiple Correspondence Analysis and
#' Related Methods} (pp. 503--521). Chapman & Hall.
#'
#' @seealso \code{\link{gradientDesc}}
#'
#' @examples
#' \donttest{
#' data("Methylation")
#' set.seed(123456)
#'
#' # Gradient methods (default)
#' performanceBLB(xi = Methylation)
#' performanceBLB(xi = Methylation, maxit = 150)
#'
#' # Derivative-free methods
#' performanceBLB(xi = Methylation, method = 1)
#' performanceBLB(xi = Methylation, method = 1, maxit = 100)
#'
#' # Quasi-Newton methods
#' performanceBLB(xi = Methylation, method = 3)
#' performanceBLB(xi = Methylation, method = 3, maxit = 100)
#'
#' # All methods
#' performanceBLB(xi = Methylation, method = 4)
#' }

performanceBLB <- function(xi, k = 2, L = 0, method = NULL, maxit = NULL) {

  n      <- nrow(xi)
  p      <- ncol(xi)
  aik    <- n * k
  bjk    <- p * (k + 1)
  dTheta <- aik + bjk

  if (is.null(method)) method <- 2

  method0 <- switch(as.character(method),
    "1" = c("Nelder-Mead", "uobyqa", "newuoa"),
    "2" = c("CG", "Rcgmin"),
    "3" = c("BFGS", "L-BFGS-B", "nlm", "nlminb"),
    "4" = c("Nelder-Mead", "CG", "BFGS", "nlm", "nlminb",
             "Rcgmin", "uobyqa", "newuoa"),
    stop("'method' must be 1, 2, 3, or 4.")
  )

  args <- list(par = runif(dTheta), fn = J.BipLog.BIN,
               gr = Grad.BipLog.BIN, xt = xi, k = k,
               lambda = L, method = method0)
  if (!is.null(maxit)) args$itnmax <- maxit

  old_wd <- setwd(tempdir())
  on.exit({
    for (f in c(file.path(tempdir(), "badhess.txt"),
                file.path(old_wd,    "badhess.txt"))) {
      if (file.exists(f)) unlink(f, force = TRUE)
    }
    setwd(old_wd)
  }, add = TRUE)
  res <- suppressWarnings(do.call(optimx, args))

  res$result <- ifelse(
    res$convcode == 0,    "convergence",
    ifelse(res$convcode == 1, "maximum iterations reached",
    ifelse(res$convcode %in% c(10, 20, 21), "inadmissible value",
    ifelse(res$convcode == 9999, "method failed", NA))))

  out <- data.frame(
    method      = rownames(res),
    evaluat     = res$value,
    convergence = res$result,
    fevals      = res$fevals,
    time        = res$xtime,
    stringsAsFactors = FALSE
  )
  return(out)
}

#' @importFrom stats plogis

J.BipLog.BIN <- function(xt, par, k, lambda, ...) {
    #---- Para iniciar.
    x = as.matrix(xt)
    n = nrow(x)
    p = ncol(x)
    aik = n * k
    bjk = p * (k + 1)
    dTheta = aik + bjk

    A = matrix(par[1:aik], n, k)
    B = matrix(par[(aik + 1):dTheta], p, k + 1)

    lin = cbind(rep(1, n), A) %*% t(B)

    hX = plogis(lin)

    J = -sum(x * log(hX) + (1 - x) * log(1 - hX), na.rm = TRUE) + lambda *
        sum(A^2, na.rm = TRUE)/2 + lambda * sum(B[, -1]^2, na.rm = TRUE)/2
    return(J)
}

Grad.BipLog.BIN <- function(xt, par, k, lambda, ...) {
    x = as.matrix(xt)
    n = nrow(x)
    p = ncol(x)
    aik = n * k
    bjk = p * (k + 1)
    dTheta = aik + bjk

    A = matrix(par[1:aik], n, k)
    B = matrix(par[(aik + 1):dTheta], p, k + 1)

    lin = cbind(rep(1, n), A) %*% t(B)
    hX = plogis(lin)

    dA = (hX - x) %*% B[, -1] + lambda * A
    dB = t(hX - x) %*% cbind(rep(1, n), A) + lambda * cbind(rep(0, p), B[,
        -1])
    gradient = c(c(dA), c(dB))
    return(gradient)
}

Indsup.BIN <- function(xs, B, par, k, lambda, ...) {
    x = as.matrix(xs)
    n = nrow(x)
    p = ncol(x)
    aik = n * k
    dTheta = aik

    A = matrix(par[1:aik], n, k)
    Bs = as.matrix(B)
    lin = cbind(rep(1, n), A) %*% t(Bs)

    hX = plogis(lin)

    L = -sum(x * log(hX) + (1 - x) * log(1 - hX), na.rm = TRUE) + lambda *
        sum(A^2, na.rm = TRUE)/2 + lambda * sum(Bs[, -1]^2, na.rm = TRUE)/2
    return(L)
}

Indsup.GradBIN <- function(xs, B, par, k, lambda, ...) {
    x = as.matrix(xs)
    Bs = as.matrix(B)
    n = nrow(x)
    p = ncol(x)
    aik = n * k
    dTheta = aik

    A = matrix(par[1:aik], n, k)

    lin = cbind(rep(1, n), A) %*% t(Bs)
    hX = plogis(lin)

    dA = (hX - x) %*% Bs[, -1] + lambda * A
    gradient = c(dA)
    return(gradient)
}

train_miss_pattern <- function(data, K = 7){
    X <- as.matrix(data)
    out <- sapply(1:K, function(x) seq(x, length(X), by = K), simplify = FALSE)
    train <- list()
    for(i in 1:K){
        temp <- X
        temp[out[[i]]] <- NA
        train[[i]] <- temp
    }

    lout <- list(missWold = out, Xtrain = train)
    return(lout)
}


thresholds <- function(x, P, ncuts = 100){

    if (!requireNamespace("tidyr", quietly = TRUE)) {
        stop("Package \"tidyr\" needed for this function to work. Please install it.",
             call. = FALSE)
    }

    if (!requireNamespace("dplyr", quietly = TRUE)) {
        stop("Package \"dplyr\" needed for this function to work. Please install it.",
             call. = FALSE)
    }

    P <- as.matrix(P)
    if(is.null(colnames(x))) colnames(x) <- paste0('V', 1:ncol(x))

    N1 <- ceiling(sum(x, na.rm = T))
    N0 <- length(as.matrix(x)) - N1 - sum(is.na(x))
    TE <- lapply(seq(0, 1, length.out = ncuts), function(z){
        Pr <- ifelse(P>=z, 1, 0)
        c1 <- 1 - apply((Pr == 1) & (x == 1), 2, sum, na.rm=TRUE)/apply(x == 1, 2, sum, na.rm = T)
        c2 <- 1 - apply((Pr == 0) & (x == 0), 2, sum, na.rm=TRUE)/apply(x == 0, 2, sum, na.rm = T)
        TE <- 100/2 * (c1 + c2)
        lista <- list(TE)

    })
    TEp <-  data.frame(dplyr::bind_rows(TE), threshold = seq(0, 1, length.out = ncuts))

    thresholds <- TEp |>
        tidyr::pivot_longer(-threshold, names_to = "variable", values_to = "BACC") |>
        dplyr::group_by(variable) |>
        dplyr::mutate(merror = min(BACC)) |>
        dplyr::filter(BACC == merror) |>
        dplyr::mutate(row = dplyr::row_number(), nclass = max(row)) |>
        dplyr::mutate(sel = ifelse(nclass > 1 & row ==2, 1,
                                   ifelse(nclass == 1 & row ==1, 1, 0))) |>
        dplyr::filter(sel == 1) |> dplyr::ungroup() |>
        dplyr::select(variable, threshold, BACC)

    thresholds <- thresholds[match(colnames(x), thresholds$variable),]
    Pr <- matrix(NA, nrow(P), ncol(P))
    for(p in 1:ncol(P)){
        Pr[,p] <- ifelse( P[,p] >= thresholds$threshold[p], 1, 0)
    }
    c1 <- 1 - sum(apply((Pr == 1) & (x == 1), 2, sum, na.rm=TRUE))/N1
    c2 <- 1 - sum(apply((Pr == 0) & (x == 0), 2, sum, na.rm=TRUE))/N0
    BACC <- round(100/2 * (c1 + c2), 2)

    out <- list(pred = Pr, thres = thresholds, BACC = BACC)
}


crossval <- function(x, k = 2, K = 7, thres = NULL, method = NULL, type = NULL){

    folds <- train_miss_pattern(x, K = K)
    missVal <- folds$missWold
    train <- folds$Xtrain

    err_T = list()
    cv_errD = list()
    for(i in 1:K){
        if(method == "CG" & k > 0){
            missBip <- BiplotML::LogBip(train[[i]], k = k,
                                        method = method, type = type, plot = FALSE)
            P <- fitted_LB(missBip, type = "response")
        }else if(method == "BFGS" & k > 0){
            missBip <- BiplotML::LogBip(train[[i]], k = k,
                                        method = method, plot = FALSE)
            P <- fitted_LB(missBip, type = "response")
        }else if(method == "MM" & k > 0){
            missBip <- BiplotML::LogBip(train[[i]], k = k,
                                        method = "MM", random_start = FALSE, plot = FALSE)
            P <- fitted_LB(missBip, type = "response")
        }else{
            P <- rep(1, nrow(x)) %*% t(as.matrix(colMeans(train[[i]], na.rm=TRUE)))
        }

        Xhat <- matrix(NA, nrow(P), ncol(P))
        for(p in 1:ncol(P)){
            Xhat[,p] <- ifelse(P[,p] >= thres$threshold[p], 1, 0)
        }
        Xhat[is.na(Xhat)] <- 0

        Xhat_pred <- Xhat[missVal[[i]]]
        xReal <- x[missVal[[i]]]

        n1 <- sum(xReal);            n1t <- sum(x)
        n0 <- length(xReal) - n1;    n0t <- length(x) - n1t

        err0 <- sum(ifelse(xReal == 0 & Xhat_pred == 1, 1, 0))/n0
        err1 <- sum(ifelse(xReal == 1 & Xhat_pred == 0, 1, 0))/n1

        err0t <- sum(ifelse(x == 0 & Xhat == 1, 1, 0))/n0t
        err1t <- sum(ifelse(x == 1 & Xhat == 0, 1, 0))/n1t

        err_T[[i]] = 100/2 * (err0t + err1t)

        cv_errD[[i]] <- 100/2 * (err0 + err1)
    }
    cvT <- round(mean(sapply(err_T, mean, na.rm = TRUE), na.rm = TRUE), 2)
    cvD <- round(mean(sapply(cv_errD, mean, na.rm = TRUE), na.rm = TRUE), 2)

    out <- list(cvT = cvT, cvD = cvD)
    return(out)
}

log_like <- function(x, w = NULL, theta){
  n <- nrow(x)
  p <- ncol(x)

  if (!is.null(w)) {
    w = matrix(rep(1, n*p), n, p)
  }
  e_log_like <- matrix(NA, nrow = n, ncol = p)
  P <- plogis(theta)

  for (i in 1:nrow(x)) {
    for (j in 1:ncol(x)) {
      e_log_like[i,j] <- w[i,j] * (x[i,j] * log(P[i, j]) + (1 - x[i,j]) * log(1 - P[i, j]))
    }
  }
  log_like <- -sum(e_log_like)
  return(log_like)
}

.run_optimr <- function(par, fn, gr, ..., method, ctrl = list()) {
  old_wd <- setwd(tempdir())
  on.exit({
    for (f in c(file.path(tempdir(), "badhess.txt"),
                file.path(old_wd,    "badhess.txt"))) {
      if (file.exists(f)) unlink(f, force = TRUE)
    }
    setwd(old_wd)
  }, add = TRUE)

  if (length(ctrl) > 0) {
    suppressWarnings(
      optimr(par = par, fn = fn, gr = gr, ...,
             method = method, control = ctrl)
    )
  } else {
    suppressWarnings(
      optimr(par = par, fn = fn, gr = gr, ...,
             method = method)
    )
  }
}

# Internal: redirect ALL console output (stdout + messages) to a temp file.
# Used to suppress the verbose "controls added:" messages printed by optimx
# when using the CG method. NOTE: never call return() inside the expression
# passed to .silently() — it would bypass the finally block and leave sinks open.
.silently <- function(expr) {
  tmp <- tempfile()
  con <- file(tmp, open = "wt")
  sink(con, type = "output",  append = FALSE)
  sink(con, type = "message", append = TRUE)
  tryCatch(
    { result <- force(expr); result },
    error = function(e) { stop(e) },
    finally = {
      try(sink(type = "output"),  silent = TRUE)
      try(sink(type = "message"), silent = TRUE)
      try(close(con),             silent = TRUE)
      try(unlink(tmp),            silent = TRUE)
    }
  )
}

utils::globalVariables(c("rowId", "Dim1", "Dimb1", "Dim2", "Dimb2", "ind", "resample", "Dim1c", "Dim2c",
                         "b0", "bb0", "b1", "bb1", "b2", "bb2", "bj0c", "bj1c", "bj2c",
                         "resampleb", "x.50", "x.75", "y.50", "y.75",
                         "threshold", "variable", "cv-error", "merror",
                         "thresholds", "error", "type.error", "nclass", "sel"))

#' @importFrom stats IQR
#'
#' @export
#' @title Plot a Binary Logistic Biplot
#' @description
#' Produces a \pkg{ggplot2}-based logistic biplot from a \code{BiplotML}
#' object fitted with \code{\link{LogBip}}. Supports coloring and
#' shaping of row markers by a categorical variable, filled arrowheads,
#' dashed reference lines that span the full plot area, and flexible axis-limit control via
#' \code{xylim}, \code{xlim}, and \code{ylim}.
#'
#' @details
#' Variable vectors are drawn as arrows from the point where the predicted
#' probability equals 0.5 to the point where it equals \code{endsegm}.
#' Short arrows indicate a rapid increase in the probability of the
#' corresponding characteristic. The orthogonal projection of a row marker
#' onto a variable's arrow approximates the probability that the
#' characteristic is present for that individual.
#'
#' The three arguments that control axis limits are evaluated in the
#' following order of priority:
#' \enumerate{
#'   \item \code{xlim} and \code{ylim} (independent limits for each axis).
#'   \item \code{xylim} (symmetric limits applied to both axes).
#'   \item Automatic limits derived from all plotted elements.
#' }
#'
#' The \code{escala} argument multiplies the row marker coordinates before
#' plotting so that they are visually comparable to the variable arrows,
#' which are expressed in the original parameter units. It only affects the
#' display, not the stored coordinates.
#'
#' @author Giovany Babativa <jgbabativam@@unal.edu.co>
#'
#' @param x An object of class \code{BiplotML}, as returned by
#'   \code{\link{LogBip}}.
#' @param dim Integer vector of length 2 specifying which dimensions to plot.
#'   Default is \code{c(1, 2)}.
#' @param col.ind Optional vector of the same length as the number of rows in
#'   the original data, used to color \emph{and} shape the row markers by a
#'   categorical variable (e.g., \code{col.ind = df$group}). Levels are
#'   mapped to the \code{"Set1"} palette and to filled geometric shapes.
#'   If \code{NULL} (default), all row markers are drawn as gold triangles
#'   (\code{shape = 17}, color \code{"#E7B800"}) when no \code{col.ind} is
#'   provided.
#' @param col.var Color for the variable arrows. Default is \code{"#0E185F"}
#'   (dark navy).
#' @param label.ind Logical; if \code{TRUE}, row markers are labelled.
#'   Default is \code{FALSE}.
#' @param draw Which graph to draw. One of \code{"biplot"} (default, both row
#'   and column markers), \code{"ind"} (row markers only), or \code{"var"}
#'   (variable arrows only). Partial matching is supported.
#' @param titles Main title for the plot. If \code{NULL} (default), a
#'   generic title is used depending on \code{draw}.
#' @param ellipses Logical; if \code{TRUE}, bootstrap confidence ellipses are
#'   drawn around the row markers. Requires a bootstrap fit
#' @param endsegm End point of the variable arrow on the probability scale.
#'   The arrow starts at \eqn{p = 0.5} and ends at this value.
#'   Default is \code{0.75}.
#' @param repel Logical; if \code{TRUE}, overlapping variable labels are
#'   repelled using \pkg{ggrepel}. Default is \code{FALSE}.
#' @param xylim Numeric vector of length 2 specifying a symmetric range
#'   applied to both axes, e.g., \code{c(-80, 80)}. Overrides automatic
#'   limits. Takes precedence over automatic limits but is overridden by
#'   \code{xlim}/\code{ylim} if those are also supplied.
#'   Default is \code{NULL}.
#' @param xlim Numeric vector of length 2 specifying the range of the
#'   x-axis independently, e.g., \code{c(-100, 60)}. Takes precedence over
#'   \code{xylim}. Default is \code{NULL}.
#' @param ylim Numeric vector of length 2 specifying the range of the
#'   y-axis independently, e.g., \code{c(-80, 80)}. Takes precedence over
#'   \code{xylim}. Default is \code{NULL}.
#' @param escala Positive numeric scalar. Multiplicative factor applied to
#'   the row marker coordinates (\code{x$Ahat}) before plotting, so that
#'   they are on a comparable visual scale to the variable arrows.
#'   If \code{NULL} (default), the value is chosen automatically so that
#'   the range of the scaled row markers matches the range of the variable
#'   arrows, producing a visually balanced biplot. Pass an explicit numeric
#'   value to override the automatic calculation (e.g., \code{escala = 65}).
#'
#' @return A \code{ggplot2} object that can be further customised with
#'   standard \pkg{ggplot2} functions (e.g., \code{theme()}, \code{labs()}).
#'
#' @references
#' Meulman, J. J., & Heiser, W. J. (1983). \emph{The Display of Bootstrap
#' Solutions in Multidimensional Scaling} (Technical memorandum). Bell
#' Laboratories.
#'
#' Vicente-Villardon, J. L., & Galindo, M. P. (2006). Logistic biplots. In
#' M. Greenacre & J. Blasius (Eds.), \emph{Multiple Correspondence Analysis
#' and Related Methods} (pp. 503--521). Chapman & Hall.
#'
#' @seealso \code{\link{LogBip}}
#'
#' @examples
#' \donttest{
#' data("Methylation")
#' set.seed(123456)
#' res <- LogBip(x = Methylation, method = "MM", maxit = 1000, plot = FALSE)
#' }

plotBLB <- function(x,
                    dim      = c(1, 2),
                    col.ind  = NULL,
                    col.var  = "#0E185F",
                    label.ind = FALSE,
                    draw     = c("biplot", "ind", "var"),
                    titles   = NULL,
                    ellipses = FALSE,
                    endsegm  = 0.75,
                    repel    = FALSE,
                    xylim    = NULL,
                    xlim     = NULL,
                    ylim     = NULL,
                    escala   = NULL) {

  for (pkg in c("ggplot2", "dplyr")) {
    if (!requireNamespace(pkg, quietly = TRUE)) {
      stop("Package '", pkg, "' is required. Please install it.",
           call. = FALSE)
    }
  }

  # ── Extract fitting objects ────────────────────────────────────────────────
  EspA <- x$Ahat
  EspB <- x$Bhat
  k    <- ncol(EspA)

  if (k < max(dim)) {
    stop("The logistic biplot has only ", k,
         " dimension(s). Reduce the requested plot dimensions.")
  }

  grap <- match.arg(draw[1], c("ind", "var", "biplot"))

  # ── Dimension column names ─────────────────────────────────────────────────
  colnames(EspA) <- paste0("Dim", seq_len(k))
  d1  <- paste0("Dim", dim[1])
  d2  <- paste0("Dim", dim[2])
  dd1 <- paste0("bb",  dim[1])
  dd2 <- paste0("bb",  dim[2])

  # ── Column marker (variable) coordinates ──────────────────────────────────
  # Arrow starts at p = 0.5 and ends at p = endsegm
  denom <- rowSums(EspB[, c(dd1, dd2), drop = FALSE] *
                     EspB[, c(dd1, dd2), drop = FALSE])
  lp <- log(endsegm / (1 - endsegm))

  EspB[["x.50"]]  <- (-EspB[["bb0"]] * EspB[[dd1]]) / denom
  EspB[["y.50"]]  <- (-EspB[["bb0"]] * EspB[[dd2]]) / denom
  EspB[["x.end"]] <- (lp - EspB[["bb0"]]) * EspB[[dd1]] / denom
  EspB[["y.end"]] <- (lp - EspB[["bb0"]]) * EspB[[dd2]] / denom

  EspB[["label"]] <- rownames(EspB)
  EspA[["label"]] <- rownames(EspA)

  # ── Row marker scaled coordinates ─────────────────────────────────────────
  if (is.null(escala)) {
    range_ind <- max(
      diff(range(c(EspA[[d1]], EspA[[d2]]), na.rm = TRUE)),
      1e-6
    )
    range_var <- diff(range(
      c(EspB[["x.50"]], EspB[["x.end"]],
        EspB[["y.50"]], EspB[["y.end"]]),
      na.rm = TRUE
    ))
    escala <- range_var / range_ind
  }

  EspA[["x.sc"]] <- EspA[[d1]] * escala
  EspA[["y.sc"]] <- EspA[[d2]] * escala

  # ── Axis limits ────────────────────────────────────────────────────────────
  # Priority: xlim/ylim > xylim > automatic
  if (!is.null(xlim) && !is.null(ylim)) {
    x.lim <- xlim
    y.lim <- ylim
  } else if (!is.null(xylim)) {
    x.lim <- xylim
    y.lim <- xylim
  } else {
    # Automatic: derive from all plotted elements
    if (ellipses && !is.null(x$Ellip)) {
      all_x <- c(EspA[["x.sc"]], EspB[["x.50"]], EspB[["x.end"]],
                 x$Ellip[[d1]])
      all_y <- c(EspA[["y.sc"]], EspB[["y.50"]], EspB[["y.end"]],
                 x$Ellip[[d2]])
    } else {
      all_x <- c(EspA[["x.sc"]], EspB[["x.50"]], EspB[["x.end"]])
      all_y <- c(EspA[["y.sc"]], EspB[["y.50"]], EspB[["y.end"]])
    }
    margin <- 0.05
    x.lim  <- range(all_x, na.rm = TRUE) * c(1 + margin, 1 + margin)
    y.lim  <- range(all_y, na.rm = TRUE) * c(1 + margin, 1 + margin)
    # Symmetric auto limits: use the wider range for both axes so
    # coord_fixed() does not distort the biplot geometry
    sym    <- max(abs(c(x.lim, y.lim)))
    x.lim  <- c(-sym, sym)
    y.lim  <- c(-sym, sym)
  }

  # ── Subtitle from method ───────────────────────────────────────────────────
  if (x$method == "CG") {
    subt <- "Estimation with Conjugate Gradient"
  } else {
    subt <- paste0("Estimation with ", x$method, " algorithm")
  }

  # ── Default titles per draw mode ───────────────────────────────────────────
  if (is.null(titles)) {
    titulo <- switch(grap,
                     ind    = "Individuals plot",
                     var    = "Variables plot",
                     biplot = "Logistic Biplot")
  } else {
    titulo <- titles
  }

  make_arrow <- function(angle = 20, len = 0.3) {
    grid::arrow(angle  = angle,
                type   = "closed",
                ends   = "last",
                length = grid::unit(len, "cm"))
  }

  ref_lines <- list(
    ggplot2::geom_vline(xintercept = 0, color = "black",
                        linetype = "dashed", linewidth = 0.4),
    ggplot2::geom_hline(yintercept = 0, color = "black",
                        linetype = "dashed", linewidth = 0.4)
  )

  g <- ggplot2::ggplot() +
    ggplot2::theme_bw() +
    ref_lines +
    ggplot2::xlim(x.lim[1], x.lim[2]) +
    ggplot2::ylim(y.lim[1], y.lim[2]) +
    ggplot2::xlab(paste("Dimension", dim[1])) +
    ggplot2::ylab(paste("Dimension", dim[2])) +
    ggplot2::labs(title = titulo, subtitle = subt) +
    ggplot2::theme(
      axis.text     = ggplot2::element_text(face = "bold"),
      plot.title    = ggplot2::element_text(hjust = 0.5),
      plot.subtitle = ggplot2::element_text(hjust = 0.5)
    )

  # ══════════════════════════════════════════════════════════════════════════
  # IND plot
  # ══════════════════════════════════════════════════════════════════════════
  if (grap == "ind") {

    if (is.null(col.ind)) {
      g <- g +
        ggplot2::geom_point(
          data  = EspA,
          ggplot2::aes(x = .data[["x.sc"]], y = .data[["y.sc"]]),
          color = "#E7B800", size = 2, shape = 17
        ) +
        ggplot2::theme(legend.position = "none")
    } else {
      g <- g +
        ggplot2::geom_point(
          data  = EspA,
          ggplot2::aes(x      = .data[["x.sc"]],
                       y      = .data[["y.sc"]],
                       colour = col.ind,
                       shape  = col.ind),
          size = 2
        ) +
        ggplot2::scale_color_brewer(palette = "Set1") +
        ggplot2::scale_shape_manual(
          values = c(16, 17, 15, 18, 19, 3, 4, 8)[
            seq_along(levels(factor(col.ind)))]
        )
    }

    if (label.ind) {
      if (repel && requireNamespace("ggrepel", quietly = TRUE)) {
        g <- g + ggrepel::geom_text_repel(
          data  = EspA,
          ggplot2::aes(x     = .data[["x.sc"]],
                       y     = .data[["y.sc"]],
                       label = .data[["label"]]),
          size          = 3.5,
          segment.color = "grey50"
        )
      } else {
        g <- g + ggplot2::geom_text(
          data  = EspA,
          ggplot2::aes(x     = .data[["x.sc"]],
                       y     = .data[["y.sc"]],
                       label = .data[["label"]]),
          size = 3.5
        )
      }
    }
  }

  # ══════════════════════════════════════════════════════════════════════════
  # VAR plot
  # ══════════════════════════════════════════════════════════════════════════
  if (grap == "var") {

    g <- g +
      ggplot2::geom_segment(
        data     = EspB,
        ggplot2::aes(x    = .data[["x.50"]],
                     y    = .data[["y.50"]],
                     xend = .data[["x.end"]],
                     yend = .data[["y.end"]]),
        arrow       = make_arrow(angle = 25),
        colour      = col.var,
        lineend     = "butt",
        linejoin    = "mitre",
        show.legend = FALSE
      )

    if (repel && requireNamespace("ggrepel", quietly = TRUE)) {
      g <- g + ggrepel::geom_text_repel(
        data  = EspB,
        ggplot2::aes(x     = .data[["x.end"]],
                     y     = .data[["y.end"]],
                     label = .data[["label"]],
                     angle = (180 / pi) * atan(.data[["y.end"]] /
                                                 .data[["x.end"]]),
                     hjust = (1 - 2 * sign(.data[["x.end"]])) / 2),
        size          = 3,
        segment.color = "grey50",
        colour        = "black",
        vjust         = 0
      )
    } else {
      g <- g + ggplot2::geom_text(
        data  = EspB,
        ggplot2::aes(x     = .data[["x.end"]],
                     y     = .data[["y.end"]],
                     label = .data[["label"]],
                     angle = (180 / pi) * atan(.data[["y.end"]] /
                                                 .data[["x.end"]]),
                     hjust = (1 - 2 * sign(.data[["x.end"]])) / 2),
        size   = 3,
        colour = "black",
        vjust  = 0
      )
    }
  }

  # ══════════════════════════════════════════════════════════════════════════
  # BIPLOT
  # ══════════════════════════════════════════════════════════════════════════
  if (grap == "biplot") {

    # ── Row markers ───────────────────────────────────────────────────────
    if (is.null(col.ind)) {
      g <- g +
        ggplot2::geom_point(
          data  = EspA,
          ggplot2::aes(x = .data[["x.sc"]], y = .data[["y.sc"]]),
          color = "#E7B800", size = 2, shape = 17
        ) +
        ggplot2::theme(legend.position = "none")
    } else {
      g <- g +
        ggplot2::geom_point(
          data  = EspA,
          ggplot2::aes(x      = .data[["x.sc"]],
                       y      = .data[["y.sc"]],
                       colour = col.ind,
                       shape  = col.ind),
          size = 2
        ) +
        ggplot2::scale_color_brewer(palette = "Set1") +
        ggplot2::scale_shape_manual(
          values = c(16, 17, 15, 18, 19, 3, 4, 8)[
            seq_along(levels(factor(col.ind)))]
        )
    }

    # ── Variable arrows ───────────────────────────────────────────────────
    g <- g +
      ggplot2::geom_segment(
        data     = EspB,
        ggplot2::aes(x    = .data[["x.50"]],
                     y    = .data[["y.50"]],
                     xend = .data[["x.end"]],
                     yend = .data[["y.end"]]),
        arrow       = make_arrow(angle = 20),
        colour      = col.var,
        lineend     = "butt",
        linejoin    = "mitre",
        show.legend = FALSE
      )

    # ── Variable labels ───────────────────────────────────────────────────
    if (repel && requireNamespace("ggrepel", quietly = TRUE)) {
      g <- g + ggrepel::geom_text_repel(
        data  = EspB,
        ggplot2::aes(x     = .data[["x.end"]],
                     y     = .data[["y.end"]],
                     label = .data[["label"]],
                     angle = (180 / pi) * atan(.data[["y.end"]] /
                                                 .data[["x.end"]]),
                     hjust = (1 - 2 * sign(.data[["x.end"]])) / 2),
        size          = 3,
        segment.color = "grey50",
        vjust         = 0
      )
    } else {
      g <- g + ggplot2::geom_text(
        data  = EspB,
        ggplot2::aes(x     = .data[["x.end"]],
                     y     = .data[["y.end"]],
                     label = .data[["label"]],
                     angle = (180 / pi) * atan(.data[["y.end"]] /
                                                 .data[["x.end"]]),
                     hjust = (1 - 1.0 * sign(.data[["x.end"]])) / 2),
        size  = 3,
        vjust = 0
      )
    }

    # ── Row labels ────────────────────────────────────────────────────────
    if (label.ind) {
      if (requireNamespace("ggrepel", quietly = TRUE)) {
        g <- g + ggrepel::geom_text_repel(
          data  = EspA,
          ggplot2::aes(x     = .data[["x.sc"]],
                       y     = .data[["y.sc"]],
                       label = .data[["label"]]),
          size          = 3.5,
          segment.color = "grey50"
        )
      } else {
        g <- g + ggplot2::geom_text(
          data  = EspA,
          ggplot2::aes(x     = .data[["x.sc"]],
                       y     = .data[["y.sc"]],
                       label = .data[["label"]]),
          size  = 3.5,
          vjust = -0.5
        )
      }
    }
  }

  # ══════════════════════════════════════════════════════════════════════════
  # Bootstrap ellipses
  # ══════════════════════════════════════════════════════════════════════════
  if (ellipses && !is.null(x$Ellip)) {
    g <- g +
      ggplot2::geom_point(
        data  = x$Ellip,
        ggplot2::aes(x = .data[[d1]], y = .data[[d2]],
                     group = .data[["ind"]]),
        size   = 0.001,
        colour = "lightgray",
        shape  = 20
      )
  }

  return(g + ggplot2::coord_fixed())
}

#' DNA Methylation Binary Data
#'
#' A binary matrix of DNA methylation measurements for a sample of individuals.
#' Each row represents an individual and each column a CpG site; a value of 1
#' indicates methylation and 0 indicates no methylation.
#'
#' @format A binary matrix with 50 rows (individuals) and 13 columns (CpG sites).
#'
#' @source Publicly available methylation data used for illustrative purposes.
#'
#' @examples
#' data("Methylation")
#' dim(Methylation)
"Methylation"

#' @title Fitted Values for a Logistic Biplot
#' @description
#' Computes the fitted (predicted) matrix for a logistic biplot model on either
#' the logit (log-odds) scale or the probability scale.
#'
#' @author Giovany Babativa <jgbabativam@@unal.edu.co>
#'
#' @param object An object of class \code{BiplotML}, as returned by
#'   \code{\link{LogBip}}.
#' @param type Scale of the fitted values: \code{"link"} for the logit scale
#'   (log-odds) or \code{"response"} for the probability scale. Partial matching
#'   is supported.
#'
#' @return A numeric matrix of fitted values with the same dimensions as the
#'   original binary matrix.
#'
#' @examples
#' \donttest{
#' data("Methylation")
#' LB    <- LogBip(Methylation, plot = FALSE)
#' Theta <- fitted_LB(LB, type = "link")      # log-odds scale
#' Pi    <- fitted_LB(LB, type = "response")  # probability scale
#' }
#' @export fitted_LB

fitted_LB <- function(object, type = c("link", "response")) {
  type  <- match.arg(type[1], c("link", "response"))
  n     <- nrow(object$Ahat)
  theta <- as.matrix(cbind(rep(1, n), object$Ahat)) %*% t(object$Bhat)
  P     <- plogis(theta)

  if (type == "link") {
    return(theta)
  } else {
    return(P)
  }
}

#' @importFrom optimx optimr
#' @importFrom stats runif
#' @importFrom stats sd
#' @export
#' @title Fit a Binary Logistic Biplot
#' @description
#' Estimates the intercept vector \eqn{\mu}, the row-marker matrix \strong{A}, and the
#' column-marker matrix \strong{B} of a logistic biplot model using the optimization
#' algorithm selected by the user.
#'
#' @details
#' The following fitting methods are available:
#'
#' \strong{Conjugate gradient (CG):} Set \code{method = "CG"} and choose the update
#' formula via \code{type}:
#' \itemize{
#'   \item \code{type = 1} --- Fletcher--Reeves
#'   \item \code{type = 2} --- Polak--Ribiere
#'   \item \code{type = 3} --- Hestenes--Stiefel
#'   \item \code{type = 4} --- Dai--Yuan
#' }
#'
#' \strong{Coordinate descent MM:} Set \code{method = "MM"} to use the iterative
#' coordinate descent Majorization-Minimization algorithm.
#'
#' \strong{Projection-based algorithm (PDLB):} Set \code{method = "PDLB"} when the
#' binary matrix contains missing values, or when the row coordinates of new
#' (supplementary) individuals need to be estimated without refitting the model.
#' See Babativa-Marquez & Vicente-Villardon (2022) for details.
#'
#' \strong{BFGS:} Set \code{method = "BFGS"} to use the Broyden--Fletcher--
#' Goldfarb--Shanno quasi-Newton method.
#'
#' @author Giovany Babativa <jgbabativam@@unal.edu.co>
#'
#' @param x A binary matrix (or a matrix with \code{NA} values when
#'   \code{method = "PDLB"}).
#' @param k Number of dimensions. Default is \code{k = 5}.
#' @param method Fitting algorithm. One of \code{"MM"} (default), \code{"CG"},
#'   \code{"PDLB"}, or \code{"BFGS"}.
#' @param type Update formula for the conjugate gradient method: \code{1} =
#'   Fletcher--Reeves, \code{2} = Polak--Ribiere, \code{3} = Beale--Sorenson.
#'   Ignored for other methods.
#' @param plot Logical; if \code{TRUE} (default), the logistic biplot is plotted
#'   after fitting.
#' @param maxit Maximum number of iterations. Defaults to \code{100} for gradient
#'   methods and \code{500} for derivative-free methods.
#' @param endsegm End point of the variable segment on the probability scale.
#'   The segment starts at 0.5 and ends at this value. Default is \code{0.90}.
#' @param label.ind Logical; if \code{TRUE}, row points are labelled. Default is
#'   \code{FALSE}.
#' @param col.ind Color for the row markers. Passed to \code{\link{plotBLB}}.
#' @param draw Which graph to draw: \code{"biplot"} (default) for both rows and
#'   columns, \code{"ind"} for individuals only, or \code{"var"} for variables
#'   only.
#' @param random_start Logical; if \code{TRUE}, parameters are initialised
#'   randomly. If \code{FALSE} (default), an SVD-based initialisation is used.
#' @param L Ridge penalization parameter. Default is \code{L = 0} (no penalty).
#' @param cv_LogBip Logical; indicates whether the function is being called
#'   internally by \code{\link{cv_LogBip}}. Users should leave this as
#'   \code{FALSE} (default).
#'
#' @return An object of class \code{BiplotML} (a named list) containing:
#'   \describe{
#'     \item{\code{Ahat}}{Data frame of row-marker coordinates.}
#'     \item{\code{Bhat}}{Data frame of column-marker coordinates, including the
#'       intercept column \code{bb0}.}
#'     \item{\code{method}}{Character string identifying the fitting method used.}
#'     \item{\code{loss_function}}{Vector of loss-function values at each
#'       iteration (MM and PDLB methods only).}
#'     \item{\code{iterations}}{Number of iterations performed (MM and PDLB
#'       methods only).}
#'     \item{\code{impute_x}}{Imputed binary matrix (PDLB method only).}
#'   }
#'
#' @references
#' Babativa-Marquez, J. G., & Vicente-Villardon, J. L. (2026). Logistic biplot
#' with missing data. \emph{In process}.
#'
#' Babativa-Marquez, J. G., & Vicente-Villardon, J. L. (2021). Logistic biplot
#' by conjugate gradient algorithms and iterated SVD. \emph{Mathematics},
#' \emph{9}(16), 2015. \doi{10.3390/math9162015}
#'
#' Nash, J. C. (2011). Unifying optimization algorithms to aid software system
#' users: optimx for R. \emph{Journal of Statistical Software}, \emph{43}(9),
#' 1--14.
#'
#' Nash, J. C. (2014). On best practice optimization methods in R.
#' \emph{Journal of Statistical Software}, \emph{60}(2), 1--14.
#'
#' Nocedal, J., & Wright, S. (2006). \emph{Numerical Optimization} (2nd ed.).
#' Springer.
#'
#' Vicente-Villardon, J. L., & Galindo, M. P. (2006). Logistic biplots. In
#' M. Greenacre & J. Blasius (Eds.), \emph{Multiple Correspondence Analysis and
#' Related Methods} (pp. 503--521). Chapman & Hall.
#'
#' @seealso \code{\link{plotBLB}}, \code{\link{pred_LB}}, \code{\link{fitted_LB}}
#'
#' @examples
#' \donttest{
#' data("Methylation")
#'
#' # Fit using the coordinate descent MM algorithm
#' res_MM <- LogBip(x = Methylation, method = "MM", maxit = 1000)
#'
#' # Fit using the PDLB algorithm with simulated missing data
#' set.seed(12345)
#' n <- nrow(Methylation); p <- ncol(Methylation)
#' miss <- matrix(rbinom(n * p, 1, 0.2), n, p)
#' miss <- ifelse(miss == 1, NA, miss)
#' x_miss <- Methylation + miss
#' res_PDLB <- LogBip(x = x_miss, method = "PDLB", maxit = 1000)
#' }

LogBip <- function(x, k = 5, method = "MM", type = NULL, plot = TRUE,
                   maxit = NULL, endsegm = 0.90, label.ind = FALSE,
                   col.ind = NULL,
                   draw = c("biplot", "ind", "var"),
                   random_start = FALSE, L = 0, cv_LogBip = FALSE) {

  x <- as.matrix(x)
  verify <- apply(x, 2, sd, na.rm = TRUE)

  if (any(verify == 0)) {
    stop("Some variables have zero variance; the procedure cannot be applied.")
  }

  if (cv_LogBip & any(is.na(x))) {
    x <- sweep(x, MARGIN = 2,
               STATS = colMeans(x, na.rm = TRUE),
               FUN = function(z, s) ifelse(is.na(z), s, z))
  } else if (any(is.na(x)) & method != "PDLB") {
    warning("The binary matrix contains missing values; method '", method,
            "' has been switched to 'PDLB'.")
    method <- "PDLB"
  }

  n <- nrow(x); p <- ncol(x); aik <- n * k; bjk <- p * (k + 1)
  dTheta <- aik + bjk; s <- k + 1

  truncated <- FALSE
  if (n > 100 | p > 20) {
    truncated <- TRUE
    if (!requireNamespace("RSpectra", quietly = TRUE)) {
      message("Package 'RSpectra' is recommended to speed up the algorithm.")
      truncated <- FALSE
    }
  }

  if (method != "PDLB") {
    if (random_start) {
      params <- runif(dTheta)
    } else {
      b0 <- colMeans(4 * x, na.rm = TRUE)
      if (truncated) {
        udv <- RSpectra::svds(scale(4 * x, center = b0, scale = FALSE),
                              k = k)
      } else {
        udv <- svd(scale(4 * x, center = b0, scale = FALSE))
      }
      A <- matrix(udv$u[, 1:k], n, k) %*% diag(udv$d[1:k], nrow = k, ncol = k)
      B <- matrix(udv$v[, 1:k], p, k)
      params <- c(A, b0, B)
    }
  }

  if (method == "CG") {
    res <- .silently(
      .run_optimr(par = params, fn = J.BipLog.BIN,
                  gr = Grad.BipLog.BIN,
                  xt = x, k = k, lambda = L,
                  method = method,
                  ctrl = list(type = type)))
  } else if (method == "MM") {
    if (!is.null(maxit)) {
      res <- sdv_MM(x = x, k = k, iterations = maxit,
                    random = random_start, truncated = truncated)
    } else {
      res <- sdv_MM(x = x, k = k, random = random_start,
                    truncated = truncated)
    }
  } else if (method == "PDLB") {
    if (!is.null(maxit)) {
      res <- proj_LogBip(x = x, k = k, max_iters = maxit,
                         random_start = random_start)
    } else {
      res <- proj_LogBip(x = x, k = k, random_start = random_start)
    }
  } else {
    res <- .silently(
      .run_optimr(par = params, fn = J.BipLog.BIN,
                  gr = Grad.BipLog.BIN,
                  xt = x, k = k, lambda = L,
                  method = method))
  }

  if (method %in% c("MM", "PDLB")) {
    Ahat <- data.frame(res$A)
  } else {
    par <- res$par
    Ahat <- data.frame(matrix(res$par[1:aik], n, k))
  }
  colnames(Ahat) <- paste0("Dim", seq_len(k))
  rownames(Ahat) <- rownames(x)

  if (method %in% c("MM", "PDLB")) {
    Bhat <- data.frame(res$mu, res$B)
  } else {
    Bhat <- data.frame(matrix(par[(aik + 1):dTheta], p, k + 1))
  }
  rownames(Bhat) <- colnames(x)
  colnames(Bhat) <- paste0("bb", seq(0, k, 1))

  if (method == "MM") {
    method <- "coordinate descent MM"
    out <- list(Ahat = Ahat, Bhat = Bhat, method = method,
                loss_function = res$loss_func, iterations = res$iterations)
  } else if (method == "PDLB") {
    method <- "data projection with block coordinate descent"
    out <- list(Ahat = Ahat, Bhat = Bhat, impute_x = res$x_est,
                method = method,
                loss_function = res$loss_funct, iterations = res$iter)
  } else {
    if (method == "CG" & type == 1) method <- "CG: Fletcher--Reeves"
    if (method == "CG" & type == 2) method <- "CG: Polak--Ribiere"
    if (method == "CG" & type == 3) method <- "CG: Beale--Sorenson"
    if (method == "Rcgmin")         method <- "CG: Dai--Yuan"
    out <- list(Ahat = Ahat, Bhat = Bhat, method = method)
  }

  if (plot & ncol(Ahat) > 1) {
    print(plotBLB(x = out, ellipses = FALSE, endsegm = endsegm,
                  titles = "Logistic Biplot", label.ind = label.ind,
                  col.ind = col.ind, draw = draw))
  }

  class(out) <- c("BiplotML", "list")
  return(out)
}

#' @importFrom stats qlogis
#' @importFrom stats rbinom
#' @importFrom stats rnorm
#' @export
#' @title Simulate a Multivariate Binary Matrix
#' @description
#' Simulates a binary data matrix from a logistic biplot latent variable model
#' with known parameters, useful for benchmarking and cross-validation studies.
#'
#' @author Giovany Babativa <jgbabativam@@unal.edu.co>
#'
#' @param n Number of rows (individuals).
#' @param p Number of columns (variables).
#' @param k Number of underlying latent dimensions.
#' @param D Sparsity control: the marginal probability of a 1 in the population.
#'   A value close to 0 or 1 yields a sparse or dense matrix, respectively.
#' @param C Variance scaling factor for the row scores. Default is \code{C = 1}.
#'
#' @return A named list with components:
#'   \describe{
#'     \item{\code{X}}{Simulated binary matrix (\eqn{n \times p}).}
#'     \item{\code{P}}{Matrix of true Bernoulli probabilities (\eqn{n \times p}).}
#'     \item{\code{Theta}}{Matrix of true log-odds (natural parameters).}
#'     \item{\code{A}}{True row-marker matrix (\eqn{n \times k}).}
#'     \item{\code{B}}{True column-marker matrix (\eqn{p \times k}), orthonormal.}
#'     \item{\code{mu}}{True intercept vector of length \eqn{p}.}
#'     \item{\code{D}}{Observed proportion of ones in \strong{X}.}
#'     \item{\code{n}}{Number of rows.}
#'     \item{\code{p}}{Number of columns.}
#'   }
#'
#' @seealso \code{\link{cv_LogBip}}
#'
#' @examples
#' x <- simBin(n = 100, p = 50, k = 3, D = 0.5)

simBin <- function(n, p, k, D, C = 1) {

  for (pkg in c("pracma", "mvtnorm")) {
    if (!requireNamespace(pkg, quietly = TRUE)) {
      stop("Package '", pkg, "' is required for this function. Please install it.",
           call. = FALSE)
    }
  }

  Bp <- matrix(rnorm(p * k), p)
  B  <- pracma::gramSchmidt(Bp)$Q

  S      <- diag(k)
  mu     <- rep(qlogis(D), p)
  centro <- rep(0, k)
  A      <- mvtnorm::rmvnorm(n, mean = centro, sigma = S)

  logOdds <- rep(1, n) %*% t(mu) + C * (A %*% t(B))
  P       <- plogis(logOdds)

  X <- t(sapply(seq_len(n), function(i)
    sapply(seq_len(p), function(j) rbinom(1, 1, P[i, j]))))

  desb <- sum(X) / length(X)

  list(X = X, P = P, Theta = logOdds, A = A, B = B,
       mu = mu, D = desb, n = n, p = p)
}

#' @export
#' @title Cross-Validation for Logistic Biplot
#' @description
#' Performs k-fold cross-validation for a logistic biplot model across a range
#' of dimensions, enabling selection of the optimal number of latent dimensions.
#'
#' @author Giovany Babativa <jgbabativam@@unal.edu.co>
#'
#' @param data A binary matrix.
#' @param k Integer vector of dimensions to evaluate. Default is \code{0:5}.
#' @param K Number of folds. Default is \code{K = 7}.
#' @param method Fitting algorithm: \code{"MM"} (default), \code{"CG"},
#'   \code{"PDLB"}, or \code{"BFGS"}.
#' @param type Update formula for the CG method (see \code{\link{LogBip}}).
#' @param plot Logical; if \code{TRUE} (default), the cross-validation error
#'   curve is plotted.
#' @param maxit Maximum number of iterations. Defaults to \code{100} for
#'   gradient methods and \code{2000} for the MM algorithm.
#'
#' @return A data frame with columns \code{k}, \code{cv-error} (mean
#'   cross-validation error, in percent), and \code{train-error} (mean training
#'   error, in percent).
#'
#' @references
#' Bro, R., Kjeldahl, K., & Smilde, A. K. (2008). Cross-validation of component
#' models: a critical look at current methods. \emph{Analytical and Bioanalytical
#' Chemistry}, \emph{390}(5), 1241--1251.
#'
#' Wold, S. (1978). Cross-validatory estimation of the number of components in
#' factor and principal components models. \emph{Technometrics}, \emph{20}(4),
#' 397--405.
#'
#' @seealso \code{\link{LogBip}}, \code{\link{pred_LB}}, \code{\link{fitted_LB}},
#'   \code{\link{simBin}}
#'
#' @examples
#' \donttest{
#' set.seed(1234)
#' x <- simBin(n = 100, p = 50, k = 3, D = 0.5, C = 20)
#'
#' # Cross-validation using the MM algorithm
#' cv_MM <- cv_LogBip(data = x$X, k = 0:5, method = "MM", maxit = 1000)
#'
#' # Cross-validation using the PDLB algorithm
#' cv_PB <- cv_LogBip(data = x$X, k = 0:5, method = "PDLB", maxit = 1000)
#' }

cv_LogBip <- function(data, k = 0:5, K = 7, method = "MM", type = NULL,
                      plot = TRUE, maxit = NULL) {

  x <- as.matrix(data)

  verify <- apply(x, 2, sd, na.rm = TRUE)
  if (any(verify == 0)) {
    stop("Some variables have zero variance; the procedure cannot be applied.")
  }

  min_k <- min(k)
  k_seq <- k

  if (any(is.na(x)) & method != "PDLB") {
    warning("The binary matrix contains missing values; method '", method,
            "' has been switched to 'PDLB'.")
    method <- "PDLB"
  }

  cvD <- matrix(NA, length(k_seq), 3)

  for (k in k_seq) {
    if (method == "CG" & k > 0) {
      bip   <- .silently(
        BiplotML::LogBip(x, k = k, method = method, type = type,
                         plot = FALSE))
      thres <- BiplotML::pred_LB(bip, x, ncuts = 50)$thresholds

    } else if (method == "BFGS" & k > 0) {
      bip   <- .silently(
        BiplotML::LogBip(x, k = k, method = method, plot = FALSE))
      thres <- BiplotML::pred_LB(bip, x, ncuts = 50)$thresholds

    } else if (method == "MM" & k > 0) {
      bip <- if (!is.null(maxit)) {
        BiplotML::LogBip(x, k = k, method = "MM", maxit = maxit, plot = FALSE)
      } else {
        BiplotML::LogBip(x, k = k, method = "MM", plot = FALSE)
      }
      thres <- BiplotML::pred_LB(bip, x, ncuts = 50)$thresholds

    } else if (method == "PDLB" & k > 0) {
      bip <- if (!is.null(maxit)) {
        BiplotML::LogBip(x, k = k, method = "PDLB", maxit = maxit, plot = FALSE)
      } else {
        BiplotML::LogBip(x, k = k, method = "PDLB", plot = FALSE)
      }
      xt    <- bip$impute_x
      thres <- BiplotML::pred_LB(bip, bip$impute_x, ncuts = 50)$thresholds

    } else {
      theta <- rep(1, nrow(x)) %*% t(as.matrix(colMeans(x, na.rm = TRUE)))
      P     <- plogis(theta)
      thres <- thresholds(x = x, P = P, ncuts = 50)$thres
    }

    if (k == 0 & any(is.na(x))) {
      m  <- apply(x, 2, mean, na.rm = TRUE)
      xt <- ifelse(is.na(x) & m > 0.5, 1, 0)
    }

    if (method != "PDLB")                        xt <- x
    if (k == 0 & any(!is.na(x)) & method == "PDLB") xt <- x

    folds    <- train_miss_pattern(xt, K = K)
    missVal  <- folds$missWold
    train    <- folds$Xtrain

    cv_errD  <- list()
    train_err <- list()

    for (i in seq_len(K)) {
      if (method == "CG" & k > 0) {
        missBip <- .silently(
          BiplotML::LogBip(train[[i]], k = k, method = method,
                           type = type, plot = FALSE,
                           cv_LogBip = TRUE))
        P <- fitted_LB(missBip, type = "response")

      } else if (method == "BFGS" & k > 0) {
        missBip <- if (!is.null(maxit)) {
          .silently(BiplotML::LogBip(train[[i]], k = k, method = method,
                                     maxit = maxit, plot = FALSE,
                                     cv_LogBip = TRUE))
        } else {
          .silently(BiplotML::LogBip(train[[i]], k = k, method = method,
                                     plot = FALSE, cv_LogBip = TRUE))
        }
      } else if (method %in% c("MM", "PDLB") & k > 0) {
        missBip <- if (!is.null(maxit)) {
          BiplotML::LogBip(train[[i]], k = k, method = method, maxit = maxit,
                           plot = FALSE, cv_LogBip = TRUE)
        } else {
          BiplotML::LogBip(train[[i]], k = k, method = method, plot = FALSE,
                           cv_LogBip = TRUE)
        }
        P <- fitted_LB(missBip, type = "response")

      } else {
        theta <- rep(1, nrow(x)) %*%
          t(as.matrix(colMeans(train[[i]], na.rm = TRUE)))
        P <- plogis(theta)
      }

      Xhat <- matrix(NA, nrow(P), ncol(P))
      for (p in seq_len(ncol(P))) {
        Xhat[, p] <- ifelse(P[, p] >= thres$threshold[p], 1, 0)
      }
      Xhat[is.na(Xhat)] <- 0

      Xhat_pred <- Xhat[missVal[[i]]]
      xReal     <- xt[missVal[[i]]]

      n1  <- sum(xReal);           n1t <- sum(xt)
      n0  <- length(xReal) - n1;   n0t <- length(xt) - n1t

      err0 <- sum(ifelse(xReal == 0 & Xhat_pred == 1, 1, 0)) / n0
      err1 <- sum(ifelse(xReal == 1 & Xhat_pred == 0, 1, 0)) / n1

      err0t <- sum(ifelse(xt == 0 & Xhat == 1, 1, 0)) / n0t
      err1t <- sum(ifelse(xt == 1 & Xhat == 0, 1, 0)) / n1t

      cv_errD[[i]]   <- 100 / 2 * (err0 + err1)
      train_err[[i]] <- 100 / 2 * (err0t + err1t)
    }

    row <- k - min_k + 1
    cvD[row, 1] <- k
    cvD[row, 2] <- round(mean(sapply(cv_errD,   mean, na.rm = TRUE),
                              na.rm = TRUE), 2)
    cvD[row, 3] <- round(mean(sapply(train_err, mean, na.rm = TRUE),
                              na.rm = TRUE), 2)
  }

  out <- as.data.frame(cvD)
  colnames(out) <- c("k", "cv-error", "train-error")

  if (plot) {
    if (!requireNamespace("ggplot2", quietly = TRUE) |
        !requireNamespace("tidyr",   quietly = TRUE)) {
      message("Packages 'ggplot2' and 'tidyr' are required for plotting.")
    } else {
      cvp <- out |>
        tidyr::pivot_longer(-k, names_to = "type.error", values_to = "error") |>
        ggplot2::ggplot(ggplot2::aes(x = k, y = error,
                                     linetype = type.error,
                                     shape    = type.error,
                                     color    = type.error)) +
        ggplot2::geom_line() +
        ggplot2::geom_point(size = 3.0) +
        ggplot2::geom_vline(
          xintercept = out[which.min(out$`cv-error`), "k"],
          linetype = 2) +
        ggplot2::scale_color_manual(values = c("red", "blue")) +
        ggplot2::labs(x = "Dimensions (k)", y = "Classification error (%)",
                      caption = paste("Cross-validation using the",
                                      method, "method")) +
        ggplot2::scale_x_continuous(breaks = seq(0, 20, 1)) +
        ggplot2::theme_classic() +
        ggplot2::theme(legend.position  = "top",
                       legend.title     = ggplot2::element_blank(),
                       legend.text      = ggplot2::element_text(size = 11))
      print(cvp)
    }
  }

  return(out)
}

#' @export
#' @title Fit a Binary Logistic Biplot via Gradient Descent
#' @description
#' Estimates the row-marker matrix \strong{A} and the column-marker matrix
#' \strong{B} of a binary logistic biplot using a simple (batch) gradient
#' descent algorithm. This function is mainly provided for pedagogical purposes
#' and benchmarking; the MM and CG methods in \code{\link{LogBip}} are
#' generally faster and more reliable.
#'
#' @details
#' The model is
#' \deqn{\mathrm{logit}(\pi_{ij}) =
#'   \log\!\left(\frac{\pi_{ij}}{1-\pi_{ij}}\right) =
#'   \mu_j + \sum_{s=1}^k b_{js}\,a_{is} = \mu_j + \mathbf{a}_i^\top \mathbf{b}_j.}
#' The gradient with respect to the full parameter vector is
#' \deqn{\nabla\ell =
#'   \left(\frac{\partial\ell}{\partial\boldsymbol{\mu}},\,
#'         \frac{\partial\ell}{\partial\mathbf{A}},\,
#'         \frac{\partial\ell}{\partial\mathbf{B}}\right) =
#'   \left((\boldsymbol{\Pi}-\mathbf{X})^\top,\;
#'         (\boldsymbol{\Pi}-\mathbf{X})\mathbf{B},\;
#'         (\boldsymbol{\Pi}-\mathbf{X})^\top\mathbf{A}\right).}
#'
#' @author Giovany Babativa <jgbabativam@@unal.edu.co>
#'
#' @param x A binary matrix.
#' @param k Number of dimensions. Default is \code{k = 2}.
#' @param rate Learning rate \eqn{\alpha} for the gradient descent update.
#'   Default is \code{0.001}.
#' @param converg Convergence tolerance: the algorithm stops when the relative
#'   change in the loss function is below this value. Default is \code{0.001}.
#' @param max_iter Maximum number of iterations.
#' @param plot Logical; if \code{TRUE}, the logistic biplot is plotted after
#'   fitting. Default is \code{FALSE}.
#' @param ... Additional arguments (currently unused).
#'
#' @return An object of class \code{BiplotML} (a named list) containing:
#'   \describe{
#'     \item{\code{Ahat}}{Estimated row-marker matrix.}
#'     \item{\code{Bhat}}{Estimated column-marker matrix (including intercepts).}
#'     \item{\code{method}}{Character string \code{"Gradient Descent"}.}
#'   }
#'
#' @references
#' Vicente-Villardon, J. L., & Galindo, M. P. (2006). Logistic biplots. In
#' M. Greenacre & J. Blasius (Eds.), \emph{Multiple Correspondence Analysis and
#' Related Methods} (pp. 503--521). Chapman & Hall.
#'
#' @seealso \code{\link{plotBLB}}, \code{\link{performanceBLB}}
#'
#' @examples
#' data("Methylation")
#' set.seed(02052020)
#' outGD <- gradientDesc(x = Methylation, k = 2, max_iter = 10000, plot = TRUE)

gradientDesc <- function(x, k = 2, rate = 0.001, converg = 0.001,
                         max_iter, plot = FALSE, ...) {

  x   <- as.matrix(x)
  n   <- nrow(x); p <- ncol(x)
  aik <- n * k;   bjk <- p * (k + 1)
  dTheta <- aik + bjk

  par <- runif(dTheta)
  A   <- matrix(par[1:aik], n, k)
  B   <- matrix(par[(aik + 1):dTheta], p, k + 1)

  lin  <- cbind(rep(1, n), A) %*% t(B)
  hX   <- plogis(lin)
  J    <- -sum(x * log(hX) + (1 - x) * log(1 - hX), na.rm = TRUE)
  MSE  <- J / n

  converged  <- FALSE
  iterations <- 0

  while (!converged) {
    A_new <- A - rate * (hX - x) %*% B[, -1]
    B_new <- B - rate * t(hX - x) %*% cbind(rep(1, n), A)

    lin_new <- cbind(rep(1, n), A_new) %*% t(B_new)
    hX_hat  <- plogis(lin_new)
    J_new   <- -sum(x * log(hX_hat) + (1 - x) * log(1 - hX_hat),
                    na.rm = TRUE)
    MSE_new <- J_new / n

    iterations <- iterations + 1

    if (abs(MSE_new - MSE) <= converg) {
      converged <- TRUE
    }

    if (iterations >= max_iter) {
      warning("The algorithm reached ", max_iter,
              " iterations without converging.")
      converged <- TRUE
    }

    A   <- A_new; B <- B_new
    hX  <- hX_hat
    MSE <- MSE_new
  }

  Ahat <- data.frame(A)
  Bhat <- data.frame(B)
  colnames(Ahat) <- paste0("Dim", seq_len(k))
  colnames(Bhat) <- paste0("bb",  seq(0, k, 1))
  rownames(Ahat) <- rownames(x)
  rownames(Bhat) <- colnames(x)

  out <- list(Ahat = Ahat, Bhat = Bhat, method = "Gradient Descent")

  if (plot & ncol(Ahat) > 1) {
    print(plotBLB(x = out, ellipses = FALSE,
                  titles = "Logistic Biplot - Gradient Descent"))
  }

  class(out) <- c("BiplotML", "list")
  return(out)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict_LB.R
\name{pred_LB}
\alias{pred_LB}
\title{Predict Binary Responses from a Logistic Biplot}
\usage{
pred_LB(object, x, ncuts = 100)
}
\arguments{
\item{object}{An object of class \code{BiplotML}, as returned by
\code{\link{LogBip}}.}

\item{x}{The original binary matrix used to fit the model.}

\item{ncuts}{Number of equally spaced threshold candidates in \eqn{[0, 1]}.
Default is \code{100}.}
}
\value{
A named list of class \code{BiplotML} with components:
  \describe{
    \item{\code{thresholds}}{Data frame with the optimal threshold and minimum
      BER for each variable.}
    \item{\code{predictX}}{Predicted binary matrix.}
    \item{\code{fitted}}{Confusion matrix (sensitivity, specificity, global
      accuracy) for each variable.}
    \item{\code{BER}}{Overall Balanced Error Rate (in percent).}
  }
}
\description{
Predicts the binary response matrix from a fitted logistic biplot and
computes the optimal classification threshold for each variable by minimising
the Balanced Error Rate (BER).
}
\details{
The optimal threshold for variable \eqn{j} is the value \eqn{\alpha_j \in [0,1]}
that minimises the Balanced Error Rate:
\deqn{BER_j = 1 - \frac{1}{2}
  \left(\frac{TP_j}{TP_j + FN_j} + \frac{TN_j}{TN_j + FP_j}\right),}
where \eqn{TP}, \eqn{TN}, \eqn{FP}, and \eqn{FN} denote true positives,
true negatives, false positives, and false negatives, respectively.
}
\examples{
\donttest{
data("Methylation")
LB  <- LogBip(Methylation, plot = FALSE)
out <- pred_LB(LB, Methylation)
}
}
\author{
Giovany Babativa <jgbabativam@unal.edu.co>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitted_LB.R
\name{fitted_LB}
\alias{fitted_LB}
\title{Fitted Values for a Logistic Biplot}
\usage{
fitted_LB(object, type = c("link", "response"))
}
\arguments{
\item{object}{An object of class \code{BiplotML}, as returned by
\code{\link{LogBip}}.}

\item{type}{Scale of the fitted values: \code{"link"} for the logit scale
(log-odds) or \code{"response"} for the probability scale. Partial matching
is supported.}
}
\value{
A numeric matrix of fitted values with the same dimensions as the
  original binary matrix.
}
\description{
Computes the fitted (predicted) matrix for a logistic biplot model on either
the logit (log-odds) scale or the probability scale.
}
\examples{
\donttest{
data("Methylation")
LB    <- LogBip(Methylation, plot = FALSE)
Theta <- fitted_LB(LB, type = "link")      # log-odds scale
Pi    <- fitted_LB(LB, type = "response")  # probability scale
}
}
\author{
Giovany Babativa <jgbabativam@unal.edu.co>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SVD_MM.R
\name{sdv_MM}
\alias{sdv_MM}
\title{Fit a Binary Logistic Biplot via Coordinate Descent MM Algorithm}
\usage{
sdv_MM(
  x,
  k = 5,
  iterations = 1000,
  truncated = TRUE,
  random = FALSE,
  epsilon = 1e-04
)
}
\arguments{
\item{x}{A binary matrix with no missing values.}

\item{k}{Number of dimensions. Default is \code{k = 5}.}

\item{iterations}{Maximum number of iterations. Default is \code{1000}.}

\item{truncated}{Logical; if \code{TRUE} (default for large matrices), the
truncated SVD from \pkg{RSpectra} is used to speed up computation.}

\item{random}{Logical; if \code{TRUE}, parameters are initialised randomly.
Default is \code{FALSE} (SVD initialisation).}

\item{epsilon}{Convergence tolerance. The algorithm stops when the relative
decrease in the loss function is below this value. Default is \code{1e-4}.}
}
\value{
A named list with components:
  \describe{
    \item{\code{mu}}{Estimated intercept vector of length \eqn{p}.}
    \item{\code{A}}{Estimated row-marker matrix (\eqn{n \times k}).}
    \item{\code{B}}{Estimated column-marker matrix (\eqn{p \times k}).}
    \item{\code{iterations}}{Number of iterations performed.}
    \item{\code{loss_func}}{Vector of normalised loss-function values at each
      iteration.}
  }
}
\description{
Estimates the intercept vector \eqn{\mu}, the row-marker matrix \strong{A},
and the column-marker matrix \strong{B} using an iterative coordinate descent
Majorization-Minimization (MM) algorithm. This is the low-level function
called by \code{\link{LogBip}} when \code{method = "MM"}.
}
\examples{
\donttest{
data("Methylation")
out <- sdv_MM(x = Methylation)
}
}
\references{
Babativa-Marquez, J. G., & Vicente-Villardon, J. L. (2021). Logistic biplot
by conjugate gradient algorithms and iterated SVD. \emph{Mathematics},
\emph{9}(16), 2015. \doi{10.3390/math9162015}

Vicente-Villardon, J. L., & Galindo, M. P. (2006). Logistic biplots. In
M. Greenacre & J. Blasius (Eds.), \emph{Multiple Correspondence Analysis and
Related Methods} (pp. 503--521). Chapman & Hall.
}
\seealso{
\code{\link{LogBip}}, \code{\link{cv_LogBip}}
}
\author{
Giovany Babativa <jgbabativam@unal.edu.co>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proj_LogBip.R
\name{proj_LogBip}
\alias{proj_LogBip}
\title{Fit a Binary Logistic Biplot with Missing Data via Block Coordinate Descent}
\usage{
proj_LogBip(x, k = 5, max_iters = 1000, random_start = FALSE, epsilon = 1e-05)
}
\arguments{
\item{x}{A binary matrix, possibly containing \code{NA} values.}

\item{k}{Number of dimensions. Default is \code{k = 5}.}

\item{max_iters}{Maximum number of iterations. Default is \code{1000}.}

\item{random_start}{Logical; if \code{TRUE}, parameters are initialised
randomly. Default is \code{FALSE} (SVD initialisation).}

\item{epsilon}{Convergence tolerance for the relative decrease in the loss
function. Default is \code{1e-5}.}
}
\value{
A named list with components:
  \describe{
    \item{\code{mu}}{Estimated intercept vector of length \eqn{p}.}
    \item{\code{A}}{Estimated row-marker matrix (\eqn{n \times k}).}
    \item{\code{B}}{Estimated column-marker matrix (\eqn{p \times k}).}
    \item{\code{x_est}}{Imputed binary matrix (missing entries replaced by
      fitted values).}
    \item{\code{iter}}{Number of iterations performed.}
    \item{\code{loss_funct}}{Vector of normalised loss-function values at each
      iteration.}
  }
}
\description{
Estimates the intercept vector \eqn{\mu}, the row-marker matrix \strong{A},
and the column-marker matrix \strong{B} using a data-projection model with a
block coordinate descent algorithm. Missing values in the binary matrix are
imputed iteratively during model fitting. This function also allows new
individuals to be projected as supplementary rows without refitting the model,
since the row markers are derived directly from the estimated column markers.
This is the low-level function called by \code{\link{LogBip}} when
\code{method = "PDLB"}.
}
\examples{
\donttest{
data("Methylation")
set.seed(12345)
n <- nrow(Methylation); p <- ncol(Methylation)
miss <- matrix(rbinom(n * p, 1, 0.2), n, p)
miss <- ifelse(miss == 1, NA, miss)
x_miss <- Methylation + miss
out <- proj_LogBip(x = x_miss, k = 2, max_iters = 1000)
}
}
\references{
Babativa-Marquez, J. G., & Vicente-Villardon, J. L. (2026). Logistic biplot
with missing data. \emph{In process}.

Babativa-Marquez, J. G., & Vicente-Villardon, J. L. (2021). Logistic biplot
by conjugate gradient algorithms and iterated SVD. \emph{Mathematics},
\emph{9}(16), 2015. \doi{10.3390/math9162015}

Vicente-Villardon, J. L., & Galindo, M. P. (2006). Logistic biplots. In
M. Greenacre & J. Blasius (Eds.), \emph{Multiple Correspondence Analysis and
Related Methods} (pp. 503--521). Chapman & Hall.
}
\seealso{
\code{\link{LogBip}}, \code{\link{cv_LogBip}}
}
\author{
Giovany Babativa <jgbabativam@unal.edu.co>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gradientDesc.R
\name{gradientDesc}
\alias{gradientDesc}
\title{Fit a Binary Logistic Biplot via Gradient Descent}
\usage{
gradientDesc(
  x,
  k = 2,
  rate = 0.001,
  converg = 0.001,
  max_iter,
  plot = FALSE,
  ...
)
}
\arguments{
\item{x}{A binary matrix.}

\item{k}{Number of dimensions. Default is \code{k = 2}.}

\item{rate}{Learning rate \eqn{\alpha} for the gradient descent update.
Default is \code{0.001}.}

\item{converg}{Convergence tolerance: the algorithm stops when the relative
change in the loss function is below this value. Default is \code{0.001}.}

\item{max_iter}{Maximum number of iterations.}

\item{plot}{Logical; if \code{TRUE}, the logistic biplot is plotted after
fitting. Default is \code{FALSE}.}

\item{...}{Additional arguments (currently unused).}
}
\value{
An object of class \code{BiplotML} (a named list) containing:
  \describe{
    \item{\code{Ahat}}{Estimated row-marker matrix.}
    \item{\code{Bhat}}{Estimated column-marker matrix (including intercepts).}
    \item{\code{method}}{Character string \code{"Gradient Descent"}.}
  }
}
\description{
Estimates the row-marker matrix \strong{A} and the column-marker matrix
\strong{B} of a binary logistic biplot using a simple (batch) gradient
descent algorithm. This function is mainly provided for pedagogical purposes
and benchmarking; the MM and CG methods in \code{\link{LogBip}} are
generally faster and more reliable.
}
\details{
The model is
\deqn{\mathrm{logit}(\pi_{ij}) =
  \log\!\left(\frac{\pi_{ij}}{1-\pi_{ij}}\right) =
  \mu_j + \sum_{s=1}^k b_{js}\,a_{is} = \mu_j + \mathbf{a}_i^\top \mathbf{b}_j.}
The gradient with respect to the full parameter vector is
\deqn{\nabla\ell =
  \left(\frac{\partial\ell}{\partial\boldsymbol{\mu}},\,
        \frac{\partial\ell}{\partial\mathbf{A}},\,
        \frac{\partial\ell}{\partial\mathbf{B}}\right) =
  \left((\boldsymbol{\Pi}-\mathbf{X})^\top,\;
        (\boldsymbol{\Pi}-\mathbf{X})\mathbf{B},\;
        (\boldsymbol{\Pi}-\mathbf{X})^\top\mathbf{A}\right).}
}
\examples{
data("Methylation")
set.seed(02052020)
outGD <- gradientDesc(x = Methylation, k = 2, max_iter = 10000, plot = TRUE)
}
\references{
Vicente-Villardon, J. L., & Galindo, M. P. (2006). Logistic biplots. In
M. Greenacre & J. Blasius (Eds.), \emph{Multiple Correspondence Analysis and
Related Methods} (pp. 503--521). Chapman & Hall.
}
\seealso{
\code{\link{plotBLB}}, \code{\link{performanceBLB}}
}
\author{
Giovany Babativa <jgbabativam@unal.edu.co>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Methylation.R
\docType{data}
\name{Methylation}
\alias{Methylation}
\title{DNA Methylation Binary Data}
\format{
A binary matrix with 50 rows (individuals) and 13 columns (CpG sites).
}
\source{
Publicly available methylation data used for illustrative purposes.
}
\usage{
Methylation
}
\description{
A binary matrix of DNA methylation measurements for a sample of individuals.
Each row represents an individual and each column a CpG site; a value of 1
indicates methylation and 0 indicates no methylation.
}
\examples{
data("Methylation")
dim(Methylation)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotBLB.R
\name{plotBLB}
\alias{plotBLB}
\title{Plot a Binary Logistic Biplot}
\usage{
plotBLB(
  x,
  dim = c(1, 2),
  col.ind = NULL,
  col.var = "#0E185F",
  label.ind = FALSE,
  draw = c("biplot", "ind", "var"),
  titles = NULL,
  ellipses = FALSE,
  endsegm = 0.75,
  repel = FALSE,
  xylim = NULL,
  xlim = NULL,
  ylim = NULL,
  escala = NULL
)
}
\arguments{
\item{x}{An object of class \code{BiplotML}, as returned by
\code{\link{LogBip}}.}

\item{dim}{Integer vector of length 2 specifying which dimensions to plot.
Default is \code{c(1, 2)}.}

\item{col.ind}{Optional vector of the same length as the number of rows in
the original data, used to color \emph{and} shape the row markers by a
categorical variable (e.g., \code{col.ind = df$group}). Levels are
mapped to the \code{"Set1"} palette and to filled geometric shapes.
If \code{NULL} (default), all row markers are drawn as gold triangles
(\code{shape = 17}, color \code{"#E7B800"}) when no \code{col.ind} is
provided.}

\item{col.var}{Color for the variable arrows. Default is \code{"#0E185F"}
(dark navy).}

\item{label.ind}{Logical; if \code{TRUE}, row markers are labelled.
Default is \code{FALSE}.}

\item{draw}{Which graph to draw. One of \code{"biplot"} (default, both row
and column markers), \code{"ind"} (row markers only), or \code{"var"}
(variable arrows only). Partial matching is supported.}

\item{titles}{Main title for the plot. If \code{NULL} (default), a
generic title is used depending on \code{draw}.}

\item{ellipses}{Logical; if \code{TRUE}, bootstrap confidence ellipses are
drawn around the row markers. Requires a bootstrap fit}

\item{endsegm}{End point of the variable arrow on the probability scale.
The arrow starts at \eqn{p = 0.5} and ends at this value.
Default is \code{0.75}.}

\item{repel}{Logical; if \code{TRUE}, overlapping variable labels are
repelled using \pkg{ggrepel}. Default is \code{FALSE}.}

\item{xylim}{Numeric vector of length 2 specifying a symmetric range
applied to both axes, e.g., \code{c(-80, 80)}. Overrides automatic
limits. Takes precedence over automatic limits but is overridden by
\code{xlim}/\code{ylim} if those are also supplied.
Default is \code{NULL}.}

\item{xlim}{Numeric vector of length 2 specifying the range of the
x-axis independently, e.g., \code{c(-100, 60)}. Takes precedence over
\code{xylim}. Default is \code{NULL}.}

\item{ylim}{Numeric vector of length 2 specifying the range of the
y-axis independently, e.g., \code{c(-80, 80)}. Takes precedence over
\code{xylim}. Default is \code{NULL}.}

\item{escala}{Positive numeric scalar. Multiplicative factor applied to
the row marker coordinates (\code{x$Ahat}) before plotting, so that
they are on a comparable visual scale to the variable arrows.
If \code{NULL} (default), the value is chosen automatically so that
the range of the scaled row markers matches the range of the variable
arrows, producing a visually balanced biplot. Pass an explicit numeric
value to override the automatic calculation (e.g., \code{escala = 65}).}
}
\value{
A \code{ggplot2} object that can be further customised with
  standard \pkg{ggplot2} functions (e.g., \code{theme()}, \code{labs()}).
}
\description{
Produces a \pkg{ggplot2}-based logistic biplot from a \code{BiplotML}
object fitted with \code{\link{LogBip}}. Supports coloring and
shaping of row markers by a categorical variable, filled arrowheads,
dashed reference lines that span the full plot area, and flexible axis-limit control via
\code{xylim}, \code{xlim}, and \code{ylim}.
}
\details{
Variable vectors are drawn as arrows from the point where the predicted
probability equals 0.5 to the point where it equals \code{endsegm}.
Short arrows indicate a rapid increase in the probability of the
corresponding characteristic. The orthogonal projection of a row marker
onto a variable's arrow approximates the probability that the
characteristic is present for that individual.

The three arguments that control axis limits are evaluated in the
following order of priority:
\enumerate{
  \item \code{xlim} and \code{ylim} (independent limits for each axis).
  \item \code{xylim} (symmetric limits applied to both axes).
  \item Automatic limits derived from all plotted elements.
}

The \code{escala} argument multiplies the row marker coordinates before
plotting so that they are visually comparable to the variable arrows,
which are expressed in the original parameter units. It only affects the
display, not the stored coordinates.
}
\examples{
\donttest{
data("Methylation")
set.seed(123456)
res <- LogBip(x = Methylation, method = "MM", maxit = 1000, plot = FALSE)
}
}
\references{
Meulman, J. J., & Heiser, W. J. (1983). \emph{The Display of Bootstrap
Solutions in Multidimensional Scaling} (Technical memorandum). Bell
Laboratories.

Vicente-Villardon, J. L., & Galindo, M. P. (2006). Logistic biplots. In
M. Greenacre & J. Blasius (Eds.), \emph{Multiple Correspondence Analysis
and Related Methods} (pp. 503--521). Chapman & Hall.
}
\seealso{
\code{\link{LogBip}}
}
\author{
Giovany Babativa <jgbabativam@unal.edu.co>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LogBip.R
\name{LogBip}
\alias{LogBip}
\title{Fit a Binary Logistic Biplot}
\usage{
LogBip(
  x,
  k = 5,
  method = "MM",
  type = NULL,
  plot = TRUE,
  maxit = NULL,
  endsegm = 0.9,
  label.ind = FALSE,
  col.ind = NULL,
  draw = c("biplot", "ind", "var"),
  random_start = FALSE,
  L = 0,
  cv_LogBip = FALSE
)
}
\arguments{
\item{x}{A binary matrix (or a matrix with \code{NA} values when
\code{method = "PDLB"}).}

\item{k}{Number of dimensions. Default is \code{k = 5}.}

\item{method}{Fitting algorithm. One of \code{"MM"} (default), \code{"CG"},
\code{"PDLB"}, or \code{"BFGS"}.}

\item{type}{Update formula for the conjugate gradient method: \code{1} =
Fletcher--Reeves, \code{2} = Polak--Ribiere, \code{3} = Beale--Sorenson.
Ignored for other methods.}

\item{plot}{Logical; if \code{TRUE} (default), the logistic biplot is plotted
after fitting.}

\item{maxit}{Maximum number of iterations. Defaults to \code{100} for gradient
methods and \code{500} for derivative-free methods.}

\item{endsegm}{End point of the variable segment on the probability scale.
The segment starts at 0.5 and ends at this value. Default is \code{0.90}.}

\item{label.ind}{Logical; if \code{TRUE}, row points are labelled. Default is
\code{FALSE}.}

\item{col.ind}{Color for the row markers. Passed to \code{\link{plotBLB}}.}

\item{draw}{Which graph to draw: \code{"biplot"} (default) for both rows and
columns, \code{"ind"} for individuals only, or \code{"var"} for variables
only.}

\item{random_start}{Logical; if \code{TRUE}, parameters are initialised
randomly. If \code{FALSE} (default), an SVD-based initialisation is used.}

\item{L}{Ridge penalization parameter. Default is \code{L = 0} (no penalty).}

\item{cv_LogBip}{Logical; indicates whether the function is being called
internally by \code{\link{cv_LogBip}}. Users should leave this as
\code{FALSE} (default).}
}
\value{
An object of class \code{BiplotML} (a named list) containing:
  \describe{
    \item{\code{Ahat}}{Data frame of row-marker coordinates.}
    \item{\code{Bhat}}{Data frame of column-marker coordinates, including the
      intercept column \code{bb0}.}
    \item{\code{method}}{Character string identifying the fitting method used.}
    \item{\code{loss_function}}{Vector of loss-function values at each
      iteration (MM and PDLB methods only).}
    \item{\code{iterations}}{Number of iterations performed (MM and PDLB
      methods only).}
    \item{\code{impute_x}}{Imputed binary matrix (PDLB method only).}
  }
}
\description{
Estimates the intercept vector \eqn{\mu}, the row-marker matrix \strong{A}, and the
column-marker matrix \strong{B} of a logistic biplot model using the optimization
algorithm selected by the user.
}
\details{
The following fitting methods are available:

\strong{Conjugate gradient (CG):} Set \code{method = "CG"} and choose the update
formula via \code{type}:
\itemize{
  \item \code{type = 1} --- Fletcher--Reeves
  \item \code{type = 2} --- Polak--Ribiere
  \item \code{type = 3} --- Hestenes--Stiefel
  \item \code{type = 4} --- Dai--Yuan
}

\strong{Coordinate descent MM:} Set \code{method = "MM"} to use the iterative
coordinate descent Majorization-Minimization algorithm.

\strong{Projection-based algorithm (PDLB):} Set \code{method = "PDLB"} when the
binary matrix contains missing values, or when the row coordinates of new
(supplementary) individuals need to be estimated without refitting the model.
See Babativa-Marquez & Vicente-Villardon (2022) for details.

\strong{BFGS:} Set \code{method = "BFGS"} to use the Broyden--Fletcher--
Goldfarb--Shanno quasi-Newton method.
}
\examples{
\donttest{
data("Methylation")

# Fit using the coordinate descent MM algorithm
res_MM <- LogBip(x = Methylation, method = "MM", maxit = 1000)

# Fit using the PDLB algorithm with simulated missing data
set.seed(12345)
n <- nrow(Methylation); p <- ncol(Methylation)
miss <- matrix(rbinom(n * p, 1, 0.2), n, p)
miss <- ifelse(miss == 1, NA, miss)
x_miss <- Methylation + miss
res_PDLB <- LogBip(x = x_miss, method = "PDLB", maxit = 1000)
}
}
\references{
Babativa-Marquez, J. G., & Vicente-Villardon, J. L. (2026). Logistic biplot
with missing data. \emph{In process}.

Babativa-Marquez, J. G., & Vicente-Villardon, J. L. (2021). Logistic biplot
by conjugate gradient algorithms and iterated SVD. \emph{Mathematics},
\emph{9}(16), 2015. \doi{10.3390/math9162015}

Nash, J. C. (2011). Unifying optimization algorithms to aid software system
users: optimx for R. \emph{Journal of Statistical Software}, \emph{43}(9),
1--14.

Nash, J. C. (2014). On best practice optimization methods in R.
\emph{Journal of Statistical Software}, \emph{60}(2), 1--14.

Nocedal, J., & Wright, S. (2006). \emph{Numerical Optimization} (2nd ed.).
Springer.

Vicente-Villardon, J. L., & Galindo, M. P. (2006). Logistic biplots. In
M. Greenacre & J. Blasius (Eds.), \emph{Multiple Correspondence Analysis and
Related Methods} (pp. 503--521). Chapman & Hall.
}
\seealso{
\code{\link{plotBLB}}, \code{\link{pred_LB}}, \code{\link{fitted_LB}}
}
\author{
Giovany Babativa <jgbabativam@unal.edu.co>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simBin.R
\name{simBin}
\alias{simBin}
\title{Simulate a Multivariate Binary Matrix}
\usage{
simBin(n, p, k, D, C = 1)
}
\arguments{
\item{n}{Number of rows (individuals).}

\item{p}{Number of columns (variables).}

\item{k}{Number of underlying latent dimensions.}

\item{D}{Sparsity control: the marginal probability of a 1 in the population.
A value close to 0 or 1 yields a sparse or dense matrix, respectively.}

\item{C}{Variance scaling factor for the row scores. Default is \code{C = 1}.}
}
\value{
A named list with components:
  \describe{
    \item{\code{X}}{Simulated binary matrix (\eqn{n \times p}).}
    \item{\code{P}}{Matrix of true Bernoulli probabilities (\eqn{n \times p}).}
    \item{\code{Theta}}{Matrix of true log-odds (natural parameters).}
    \item{\code{A}}{True row-marker matrix (\eqn{n \times k}).}
    \item{\code{B}}{True column-marker matrix (\eqn{p \times k}), orthonormal.}
    \item{\code{mu}}{True intercept vector of length \eqn{p}.}
    \item{\code{D}}{Observed proportion of ones in \strong{X}.}
    \item{\code{n}}{Number of rows.}
    \item{\code{p}}{Number of columns.}
  }
}
\description{
Simulates a binary data matrix from a logistic biplot latent variable model
with known parameters, useful for benchmarking and cross-validation studies.
}
\examples{
x <- simBin(n = 100, p = 50, k = 3, D = 0.5)
}
\seealso{
\code{\link{cv_LogBip}}
}
\author{
Giovany Babativa <jgbabativam@unal.edu.co>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/performanceBLB.R
\name{performanceBLB}
\alias{performanceBLB}
\title{Compare Optimization Algorithms for Binary Logistic Biplot Estimation}
\usage{
performanceBLB(xi, k = 2, L = 0, method = NULL, maxit = NULL)
}
\arguments{
\item{xi}{A binary matrix.}

\item{k}{Number of dimensions. Default is \code{k = 2}.}

\item{L}{Ridge penalization parameter. Default is \code{L = 0}.}

\item{method}{Algorithm group to compare: \code{1} (derivative-free),
\code{2} (gradient, default), \code{3} (quasi-Newton), or \code{4} (all).}

\item{maxit}{Maximum number of iterations per algorithm.}
}
\value{
A data frame with one row per algorithm and columns:
  \describe{
    \item{\code{method}}{Algorithm name.}
    \item{\code{evaluat}}{Final value of the objective function.}
    \item{\code{convergence}}{Convergence status.}
    \item{\code{fevals}}{Number of function evaluations.}
    \item{\code{time}}{Elapsed computation time.}
  }
}
\description{
Fits the binary logistic biplot model using multiple optimization algorithms
and returns a summary of their computation time, convergence status, and
number of function evaluations, facilitating algorithm selection.
}
\details{
The following algorithm groups are available via the \code{method} argument:
\itemize{
  \item \code{1} --- Derivative-free methods: Nelder-Mead, UOBYQA, NEWUOA.
  \item \code{2} --- Gradient methods (default): CG, Rcgmin.
  \item \code{3} --- Quasi-Newton methods: BFGS, L-BFGS-B, nlm, nlminb.
  \item \code{4} --- All of the above.
}
}
\examples{
\donttest{
data("Methylation")
set.seed(123456)

# Gradient methods (default)
performanceBLB(xi = Methylation)
performanceBLB(xi = Methylation, maxit = 150)

# Derivative-free methods
performanceBLB(xi = Methylation, method = 1)
performanceBLB(xi = Methylation, method = 1, maxit = 100)

# Quasi-Newton methods
performanceBLB(xi = Methylation, method = 3)
performanceBLB(xi = Methylation, method = 3, maxit = 100)

# All methods
performanceBLB(xi = Methylation, method = 4)
}
}
\references{
Nash, J. C. (2011). Unifying optimization algorithms to aid software system
users: optimx for R. \emph{Journal of Statistical Software}, \emph{43}(9),
1--14.

Nash, J. C. (2014). On best practice optimization methods in R.
\emph{Journal of Statistical Software}, \emph{60}(2), 1--14.

Vicente-Villardon, J. L., & Galindo, M. P. (2006). Logistic biplots. In
M. Greenacre & J. Blasius (Eds.), \emph{Multiple Correspondence Analysis and
Related Methods} (pp. 503--521). Chapman & Hall.
}
\seealso{
\code{\link{gradientDesc}}
}
\author{
Giovany Babativa <jgbabativam@unal.edu.co>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cv_LogBip.R
\name{cv_LogBip}
\alias{cv_LogBip}
\title{Cross-Validation for Logistic Biplot}
\usage{
cv_LogBip(
  data,
  k = 0:5,
  K = 7,
  method = "MM",
  type = NULL,
  plot = TRUE,
  maxit = NULL
)
}
\arguments{
\item{data}{A binary matrix.}

\item{k}{Integer vector of dimensions to evaluate. Default is \code{0:5}.}

\item{K}{Number of folds. Default is \code{K = 7}.}

\item{method}{Fitting algorithm: \code{"MM"} (default), \code{"CG"},
\code{"PDLB"}, or \code{"BFGS"}.}

\item{type}{Update formula for the CG method (see \code{\link{LogBip}}).}

\item{plot}{Logical; if \code{TRUE} (default), the cross-validation error
curve is plotted.}

\item{maxit}{Maximum number of iterations. Defaults to \code{100} for
gradient methods and \code{2000} for the MM algorithm.}
}
\value{
A data frame with columns \code{k}, \code{cv-error} (mean
  cross-validation error, in percent), and \code{train-error} (mean training
  error, in percent).
}
\description{
Performs k-fold cross-validation for a logistic biplot model across a range
of dimensions, enabling selection of the optimal number of latent dimensions.
}
\examples{
\donttest{
set.seed(1234)
x <- simBin(n = 100, p = 50, k = 3, D = 0.5, C = 20)

# Cross-validation using the MM algorithm
cv_MM <- cv_LogBip(data = x$X, k = 0:5, method = "MM", maxit = 1000)

# Cross-validation using the PDLB algorithm
cv_PB <- cv_LogBip(data = x$X, k = 0:5, method = "PDLB", maxit = 1000)
}
}
\references{
Bro, R., Kjeldahl, K., & Smilde, A. K. (2008). Cross-validation of component
models: a critical look at current methods. \emph{Analytical and Bioanalytical
Chemistry}, \emph{390}(5), 1241--1251.

Wold, S. (1978). Cross-validatory estimation of the number of components in
factor and principal components models. \emph{Technometrics}, \emph{20}(4),
397--405.
}
\seealso{
\code{\link{LogBip}}, \code{\link{pred_LB}}, \code{\link{fitted_LB}},
  \code{\link{simBin}}
}
\author{
Giovany Babativa <jgbabativam@unal.edu.co>
}
